# -*- coding: utf-8 -*-
"""The preprocess mediator."""

from plaso.containers import warnings
from plaso.helpers.windows import eventlog_providers
from plaso.preprocessors import logger


class PreprocessMediator(object):
  """Preprocess mediator."""

  def __init__(self, session, storage_writer, knowledge_base):
    """Initializes a preprocess mediator.

    Args:
      session (Session): session the preprocessing is part of.
      storage_writer (StorageWriter): storage writer, to store preprocessing
          information in.
      knowledge_base (KnowledgeBase): knowledge base, to fill with
          preprocessing information.
    """
    super(PreprocessMediator, self).__init__()
    self._file_entry = None
    self._knowledge_base = knowledge_base
    self._session = session
    self._storage_writer = storage_writer
    self._windows_eventlog_providers_helper = (
        eventlog_providers.WindowsEventLogProvidersHelper())
    self._windows_eventlog_providers_by_identifier = {}

  @property
  def knowledge_base(self):
    """KnowledgeBase: knowledge base."""
    return self._knowledge_base

  def AddArtifact(self, artifact_attribute_container):
    """Adds a pre-processing artifact attribute container.

    Args:
      artifact_attribute_container (ArtifactAttributeContainer): artifact
          attribute container.
    """
    if self._storage_writer:
      self._storage_writer.AddAttributeContainer(artifact_attribute_container)

  def AddEnvironmentVariable(self, environment_variable_artifact):
    """Adds an environment variable.

    Args:
      environment_variable_artifact (EnvironmentVariableArtifact): environment
          variable artifact.

    Raises:
      KeyError: if the environment variable already exists.
    """
    logger.debug('setting environment variable: {0:s} to: "{1:s}"'.format(
        environment_variable_artifact.name,
        environment_variable_artifact.value))
    self._knowledge_base.AddEnvironmentVariable(environment_variable_artifact)

    if self._storage_writer:
      self._storage_writer.AddAttributeContainer(environment_variable_artifact)

  def AddHostname(self, hostname_artifact):
    """Adds a hostname.

    Args:
      hostname_artifact (HostnameArtifact): hostname artifact.
    """
    # TODO: change storage and knowledge base to handle more than 1 hostname.
    if not self._knowledge_base.GetHostname():
      self._knowledge_base.SetHostname(hostname_artifact)

  def AddTimeZoneInformation(self, time_zone_artifact):
    """Adds a time zone defined by the operating system.

    Args:
      time_zone_artifact (TimeZoneArtifact): time zone artifact.

    Raises:
      KeyError: if the time zone already exists.
    """
    self._knowledge_base.AddAvailableTimeZone(time_zone_artifact)

  def AddUserAccount(self, user_account):
    """Adds an user account.

    Args:
      user_account (UserAccountArtifact): user account artifact.

    Raises:
      KeyError: if the user account already exists.
    """
    self._knowledge_base.AddUserAccount(user_account)

  def AddWindowsEventLogProvider(self, windows_eventlog_provider):
    """Adds a Windows EventLog provider.

    Args:
      windows_eventlog_provider (WindowsEventLogProviderArtifact): Windows
          EventLog provider.

    Raises:
      KeyError: if the Windows EventLog provider already exists.
    """
    existing_provider = None
    provider_identifier = windows_eventlog_provider.identifier

    if provider_identifier:
      existing_provider = self._windows_eventlog_providers_by_identifier.get(
          provider_identifier, None)

    if not existing_provider:
      existing_provider = self._knowledge_base.GetWindowsEventLogProvider(
          windows_eventlog_provider.log_sources[0])

    if existing_provider:
      self._windows_eventlog_providers_helper.Merge(
          existing_provider, windows_eventlog_provider)

      if self._storage_writer:
        self._storage_writer.UpdateAttributeContainer(existing_provider)

    else:
      self._windows_eventlog_providers_helper.NormalizeMessageFiles(
          windows_eventlog_provider)

      if self._storage_writer:
        self._storage_writer.AddAttributeContainer(windows_eventlog_provider)

      self._knowledge_base.AddWindowsEventLogProvider(
          windows_eventlog_provider)

      if provider_identifier:
        self._windows_eventlog_providers_by_identifier[provider_identifier] = (
            windows_eventlog_provider)

  def GetEnvironmentVariable(self, name):
    """Retrieves an environment variable.

    Args:
      name (str): name of the environment variable.

    Returns:
      EnvironmentVariableArtifact: environment variable artifact or None
          if there was no value set for the given name.
    """
    return self._knowledge_base.GetEnvironmentVariable(name)

  def ProducePreprocessingWarning(self, plugin_name, message):
    """Produces a preprocessing warning.

    Args:
      plugin_name (str): name of the preprocess plugin.
      message (str): message of the warning.
    """
    if self._storage_writer:
      path_spec = None
      if self._file_entry:
        path_spec = self._file_entry.path_spec

      warning = warnings.PreprocessingWarning(
          message=message, path_spec=path_spec, plugin_name=plugin_name)
      self._storage_writer.AddAttributeContainer(warning)

    logger.debug('[{0:s}] {1:s}'.format(plugin_name, message))

  def SetCodepage(self, codepage):
    """Sets the codepage.

    Args:
      codepage (str): codepage.

    Raises:
      ValueError: if the codepage is not supported.
    """
    if not self._knowledge_base.codepage:
      self._knowledge_base.SetCodepage(codepage)

  def SetFileEntry(self, file_entry):
    """Sets the active file entry.

    Args:
      file_entry (dfvfs.FileEntry): file entry.
    """
    self._file_entry = file_entry

  def SetLanguage(self, language):
    """Sets the language.

    Args:
      language (str): language.

    Raises:
      ValueError: if the language is not supported.
    """
    self._knowledge_base.SetLanguage(language)

  def SetTimeZone(self, time_zone):
    """Sets the time zone.

    Args:
      time_zone (str): time zone.

    Raises:
      ValueError: if the time zone is not supported.
    """
    # TODO: check if time zone is set in knowledge base.
    self._knowledge_base.SetTimeZone(time_zone)

  def SetValue(self, identifier, value):
    """Sets a value by identifier.

    Args:
      identifier (str): case insensitive unique identifier for the value.
      value (object): value.

    Raises:
      TypeError: if the identifier is not a string type.
    """
    if not self._knowledge_base.GetValue(identifier):
      self._knowledge_base.SetValue(identifier, value)



# -*- coding: utf-8 -*-
"""An output module that saves events to OpenSearch for Timesketch."""

from plaso.output import logger
from plaso.output import manager
from plaso.output import shared_opensearch


class OpenSearchTimesketchOutputModule(
    shared_opensearch.SharedOpenSearchOutputModule):
  """Output module for Timesketch OpenSearch."""

  NAME = 'opensearch_ts'
  DESCRIPTION = (
      'Saves the events into an OpenSearch database for use '
      'with Timesketch.')

  MAPPINGS_FILENAME = 'plaso.mappings'
  MAPPINGS_PATH = '/etc/timesketch'

  def __init__(self, output_mediator):
    """Initializes a Timesketch OpenSearch output module.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfvfs.
    """
    super(OpenSearchTimesketchOutputModule, self).__init__(output_mediator)
    self._timeline_identifier = None

  def _GetSanitizedEventValues(
      self, event, event_data, event_data_stream, event_tag):
    """Sanitizes the event for use in OpenSearch.

    The event values need to be sanitized to prevent certain values from
    causing problems when indexing with OpenSearch. For example the path
    specification is a nested dictionary which will cause problems for
    OpenSearch automatic indexing.

    Args:
      event (EventObject): event.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.
      event_tag (EventTag): event tag.

    Returns:
      dict[str, object]: sanitized event values.

    Raises:
      NoFormatterFound: if no event formatter can be found to match the data
          type in the event data.
    """
    event_values = (
        super(OpenSearchTimesketchOutputModule, self)._GetSanitizedEventValues(
            event=event, event_data=event_data,
            event_data_stream=event_data_stream, event_tag=event_tag))

    event_values['__ts_timeline_id'] = self._timeline_identifier

    return event_values

  def GetMissingArguments(self):
    """Retrieves a list of arguments that are missing from the input.

    Returns:
      list[str]: names of arguments that are required by the module and have
          not been specified.
    """
    if not self._timeline_identifier:
      return ['timeline_id']
    return []

  def SetTimelineIdentifier(self, timeline_identifier):
    """Sets the timeline identifier.

    Args:
      timeline_identifier (int): timeline identifier.
    """
    self._timeline_identifier = timeline_identifier
    logger.info('Timeline identifier: {0:d}'.format(self._timeline_identifier))

  def WriteHeader(self):
    """Connects to the OpenSearch server and creates the index."""
    self._Connect()

    self._CreateIndexIfNotExists(self._index_name, self._mappings)


manager.OutputManager.RegisterOutput(
    OpenSearchTimesketchOutputModule,
    disabled=shared_opensearch.opensearchpy is None)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the JSON lines output module."""

import io
import json
import os
import sys
import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory

from plaso.lib import definitions
from plaso.output import json_line

from tests import test_lib as shared_test_lib
from tests.containers import test_lib as containers_test_lib
from tests.output import test_lib


class JSONLinesOutputTest(test_lib.OutputModuleTestCase):
  """Tests for the JSON lines output module."""

  # pylint: disable=protected-access

  _OS_PATH_SPEC = path_spec_factory.Factory.NewPathSpec(
      dfvfs_definitions.TYPE_INDICATOR_OS, location='{0:s}{1:s}'.format(
          os.path.sep, os.path.join('cases', 'image.dd')))

  _TEST_EVENTS = [
      {'data_type': 'test:event',
       'hostname': 'ubuntu',
       'path_spec': path_spec_factory.Factory.NewPathSpec(
           dfvfs_definitions.TYPE_INDICATOR_TSK, inode=15,
           location='/var/log/syslog.1', parent=_OS_PATH_SPEC),
       'text': (
           'Reporter <CRON> PID: |8442| (pam_unix(cron:session): session\n '
           'closed for user root)'),
       'timestamp': '2012-06-27 18:17:01',
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN,
       'username': 'root'}]

  def testWriteHeader(self):
    """Tests the WriteHeader function."""
    test_file_object = io.StringIO()

    output_mediator = self._CreateOutputMediator()
    output_module = json_line.JSONLineOutputModule(output_mediator)
    output_module._file_object = test_file_object

    output_module.WriteHeader()

    header = test_file_object.getvalue()
    self.assertEqual(header, '')

  def testWriteFooter(self):
    """Tests the WriteFooter function."""
    test_file_object = io.StringIO()

    output_mediator = self._CreateOutputMediator()
    output_module = json_line.JSONLineOutputModule(output_mediator)
    output_module._file_object = test_file_object

    output_module.WriteFooter()

    footer = test_file_object.getvalue()
    self.assertEqual(footer, '')

  def testWriteEventBody(self):
    """Tests the WriteEventBody function."""
    test_file_object = io.StringIO()

    output_mediator = self._CreateOutputMediator()

    formatters_directory_path = self._GetTestFilePath(['formatters'])
    output_mediator.ReadMessageFormattersFromDirectory(
        formatters_directory_path)

    output_module = json_line.JSONLineOutputModule(output_mediator)
    output_module._file_object = test_file_object

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))

    output_module.WriteEventBody(event, event_data, event_data_stream, None)

    expected_timestamp = shared_test_lib.CopyTimestampFromString(
        '2012-06-27 18:17:01')

    if sys.platform.startswith('win'):
      # The dict comparison is very picky on Windows hence we
      # have to make sure the drive letter is in the same case.
      expected_os_location = os.path.abspath('\\{0:s}'.format(
          os.path.join('cases', 'image.dd')))
    else:
      expected_os_location = '{0:s}{1:s}'.format(
          os.path.sep, os.path.join('cases', 'image.dd'))

    expected_json_dict = {
        '__container_type__': 'event',
        '__type__': 'AttributeContainer',
        'date_time': {
            '__class_name__': 'PosixTimeInMicroseconds',
            '__type__': 'DateTimeValues',
            'timestamp': 1340821021000000,
        },
        'data_type': 'test:event',
        'display_name': 'TSK:/var/log/syslog.1',
        'filename': '/var/log/syslog.1',
        'hostname': 'ubuntu',
        'inode': '15',
        'message': (
            'Reporter <CRON> PID: |8442| (pam_unix(cron:session): '
            'session closed for user root)'),
        'pathspec': {
            '__type__': 'PathSpec',
            'type_indicator': 'TSK',
            'location': '/var/log/syslog.1',
            'inode': 15,
            'parent': {
                '__type__': 'PathSpec',
                'type_indicator': 'OS',
                'location': expected_os_location,
            }
        },
        'text': (
            'Reporter <CRON> PID: |8442| (pam_unix(cron:session): '
            'session\n closed for user root)'),
        'timestamp': expected_timestamp,
        'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN,
        'username': 'root',
    }
    event_body = test_file_object.getvalue()

    # We need to compare dicts since we cannot determine the order
    # of values in the string.
    json_dict = json.loads(event_body)
    self.assertEqual(json_dict, expected_json_dict)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for iOS sysdiag log files."""

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
from plaso.parsers import manager
from plaso.parsers import text_parser


class IOSSysdiagLogEventData(events.EventData):
  """iOS sysdiagnose log event data.

  Attributes:
    body (str): body of the event line.
    originating_call (str): call that created the entry.
    process_identifier (str): process_identifier.
    severity (str): severity of the message.
  """

  DATA_TYPE = 'ios:sysdiag:log:line'

  def __init__(self):
    """Initializes event data."""
    super(IOSSysdiagLogEventData, self).__init__(data_type=self.DATA_TYPE)
    self.body = None
    self.originating_call = None
    self.process_identifier = None
    self.severity = None


class IOSSysdiagLogParser(text_parser.PyparsingMultiLineTextParser):
  """Parser for iOS mobile installation log files."""

  NAME = 'ios_sysdiag_log'
  DATA_FORMAT = 'iOS sysdiag log'

  MONTHS = {
      'Jan': 1,
      'Feb': 2,
      'Mar': 3,
      'Apr': 4,
      'May': 5,
      'Jun': 6,
      'Jul': 7,
      'Aug': 8,
      'Sep': 9,
      'Oct': 10,
      'Nov': 11,
      'Dec': 12}

  ONE_OR_TWO_DIGITS = text_parser.PyparsingConstants.ONE_OR_TWO_DIGITS
  FOUR_DIGITS = text_parser.PyparsingConstants.FOUR_DIGITS
  THREE_LETTERS = text_parser.PyparsingConstants.THREE_LETTERS
  TIME_ELEMENTS = text_parser.PyparsingConstants.TIME_ELEMENTS

  _TIMESTAMP = (
      THREE_LETTERS.suppress() +
      THREE_LETTERS.setResultsName('month') +
      ONE_OR_TWO_DIGITS.setResultsName('day') + TIME_ELEMENTS +
      FOUR_DIGITS.setResultsName('year'))

  _ELEMENT_NUMBER = (
      pyparsing.Suppress('[') +
      pyparsing.Word(pyparsing.nums).setResultsName('process_identifier') +
      pyparsing.Suppress(']'))

  _ELEMENT_SEVERITY = (
      pyparsing.Suppress('<') +
      pyparsing.Word(pyparsing.alphanums).setResultsName('severity') +
      pyparsing.Suppress('>'))

  _ELEMENT_ID = (
      pyparsing.Suppress('(') +
      pyparsing.Word(pyparsing.alphanums).setResultsName('id') +
      pyparsing.Suppress(')'))

  _ELEMENT_ORIGINATOR = pyparsing.SkipTo(
      pyparsing.Literal(': ')).setResultsName('originating_call')

  _BODY_END = pyparsing.StringEnd() | _TIMESTAMP

  _ELEMENT_BODY = (
      pyparsing.Optional(pyparsing.Suppress(pyparsing.Literal(': '))) +
      pyparsing.SkipTo(_BODY_END).setResultsName('body'))

  _LINE_GRAMMAR = (
      _TIMESTAMP + _ELEMENT_NUMBER + _ELEMENT_SEVERITY +
      _ELEMENT_ID + _ELEMENT_ORIGINATOR + _ELEMENT_BODY +
      pyparsing.ZeroOrMore(pyparsing.lineEnd()))

  LINE_STRUCTURES = [('log_entry', _LINE_GRAMMAR)]

  def ParseRecord(self, parser_mediator, key, structure):
    """Parses an iOS mobile installation log record.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
        and other components, such as storage and dfvfs.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: when the structure type is unknown.
    """
    if key != 'log_entry':
      raise errors.ParseError(
          'Unable to parse record, unknown structure: {0:s}'.format(key))

    month_string = self._GetValueFromStructure(structure, 'month')

    year = self._GetValueFromStructure(structure, 'year')
    month = self.MONTHS.get(month_string)
    day = self._GetValueFromStructure(structure, 'day')
    hours = self._GetValueFromStructure(structure, 'hours')
    minutes = self._GetValueFromStructure(structure, 'minutes')
    seconds = self._GetValueFromStructure(structure, 'seconds')

    event_data = IOSSysdiagLogEventData()
    event_data.process_identifier = self._GetValueFromStructure(
        structure, 'process_identifier')
    event_data.severity = self._GetValueFromStructure(structure, 'severity')
    event_data.originating_call = self._GetValueFromStructure(
        structure, 'originating_call')
    event_data.body = self._GetValueFromStructure(structure, 'body')

    try:
      date_time = dfdatetime_time_elements.TimeElements(
          time_elements_tuple=(year, month, day, hours, minutes, seconds))
    except (TypeError, ValueError):
      parser_mediator.ProduceExtractionWarning('invalid date time value')
      return

    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_MODIFICATION)

    parser_mediator.ProduceEventWithEventData(event, event_data)

  def VerifyStructure(self, parser_mediator, lines):
    """Verifies that this is an iOS mobile installation log file.

    Args:
      parser_mediator (ParserMediator): mediates interactions between
        parsers and other components, such as storage and dfvfs.
      lines (str): one or more lines from the text file.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    match_generator = self._LINE_GRAMMAR.scanString(lines, maxMatches=1)
    return bool(list(match_generator))


manager.ParsersManager.RegisterParser(IOSSysdiagLogParser)



# -*- coding: utf-8 -*-
"""Parser for AWS ELB access logs.

This parser is based on the log format documented at
https://docs.aws.amazon.com/elasticloadbalancing/latest/application/load-balancer-access-logs.html

Note:
The AWS documentation is not clear about the meaning of the "target_port_list"
field. The assumption is that it refers to a list of possible backend instances'
IP addresses that could receive the client's request.  This parser stores the
"target_port_list" data in the "destination_list" attribute of an EventData
object.
"""

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import errors
from plaso.parsers import manager
from plaso.parsers import text_parser


class AWSELBEventData(events.EventData):
  """AWS Elastic Load Balancer access log event data.

  Attributes:
    actions_executed (str): The actions taken when processing the request.
    chosen_cert_arn (str): The ARN of the certificate
        presented to the source.
    classification (str): The classification for desync mitigation.
    classification_reason (str): The classification reason code.
    destination_group_arn (str): The Amazon Resource Name (ARN) of the
        destination group.
    destination_ip_address (str): The IP address of the destination
        that processed this request.
    destination_list (str): A space-delimited list of IP addresses
        and ports for the destinations that processed this request.
    destination_port (int): The port of the destination that processed
        this request.
    destination_processing_time (str): The total duration from
        the time the load balancer sent the request to a destination until
        the destination started to send the response headers.
    destination_status_code (int): The status code of the response
        from the destination.
    destination_status_code_list (str): A space-delimited list of status codes.
    domain_name (str): The SNI domain provided by the
        source during the TLS handshake.
    error_reason (str): The error reason code, enclosed in
        double quotes.
    matched_rule_priority (int): The priority value of the rule that
        matched the request.
    received_bytes (int): The size of the request, in bytes, received from
        the source.
    redirect_url (str): The URL of the redirect destination.
    request_processing_time (str): The total duration from
        the time the load balancer received the request until the
        time it sent the request to a destination.
    request_type (str): The type of request or connection.
    resource_identifier (str): The resource ID of the load balancer.
    response_processing_time (str): The total processing duration.
    sent_bytes (int): The size of the response, in bytes, sent to the source.
    ssl_cipher (str): The SSL cipher of the HTTPS listener.
    ssl_protocol (str): The SSL protocol of the HTTPS listener.
    source_ip_address (str): The IP address of the requesting source.
    source_port (int): The port of the requesting source.
    trace_identifier (str): The contents of the X-Amzn-Trace-Id header.
    user_agent (str): A User-Agent string.
  """

  DATA_TYPE = 'aws:elb:access'

  def __init__(self):
    """Initializes event data."""
    super(AWSELBEventData, self).__init__(data_type=self.DATA_TYPE)
    self.actions_executed = None
    self.chosen_cert_arn = None
    self.classification = None
    self.classification_reason = None
    self.destination_group_arn = None
    self.destination_ip_address = None
    self.destination_list = None
    self.destination_port = None
    self.destination_processing_time = None
    self.destination_status_code_list = None
    self.destination_status_code = None
    self.domain_name = None
    self.elb_status_code = None
    self.error_reason = None
    self.matched_rule_priority = None
    self.received_bytes = None
    self.redirect_url = None
    self.request = None
    self.request_processing_time = None
    self.request_type = None
    self.resource_identifier = None
    self.response_processing_time = None
    self.sent_bytes = None
    self.source_ip_address = None
    self.source_port = None
    self.ssl_cipher = None
    self.ssl_protocol = None
    self.trace_identifier = None
    self.user_agent = None


class AWSELBParser(text_parser.PyparsingSingleLineTextParser):
  """Parses an AWS ELB access log file."""

  NAME = 'aws_elb_access'
  DATA_FORMAT = 'AWS ELB Access log file'
  MAX_LINE_LENGTH = 3000
  _ENCODING = 'utf-8'

  BLANK = pyparsing.Literal('"-"')

  _WORD = pyparsing.Word(pyparsing.printables) | BLANK

  _QUOTE_INTEGER = (
      pyparsing.OneOrMore('"') + text_parser.PyparsingConstants.INTEGER | BLANK)

  _INTEGER = text_parser.PyparsingConstants.INTEGER | BLANK

  _FLOAT = pyparsing.Word(pyparsing.nums + '.')

  _PORT = pyparsing.Word(pyparsing.nums, max=6).setParseAction(
      text_parser.ConvertTokenToInteger) | BLANK

  _CLIENT_IP_ADDRESS_PORT = pyparsing.Group(
      text_parser.PyparsingConstants.IP_ADDRESS('source_ip_address') +
          pyparsing.Suppress(':') + _PORT('source_port') | BLANK)

  _DESTINATION_IP_ADDRESS_PORT = pyparsing.Group(
      text_parser.PyparsingConstants.IP_ADDRESS('destination_ip_address') +
          pyparsing.Suppress(':') + _PORT('destination_port') | BLANK)

  _DATE_TIME_ISOFORMAT_STRING = pyparsing.Combine(
      pyparsing.Word(pyparsing.nums, exact=4) + pyparsing.Literal('-') +
      pyparsing.Word(pyparsing.nums, exact=2) + pyparsing.Literal('-') +
      pyparsing.Word(pyparsing.nums, exact=2) + pyparsing.Literal('T') +
      pyparsing.Word(pyparsing.nums, exact=2) + pyparsing.Literal(':') +
      pyparsing.Word(pyparsing.nums, exact=2) + pyparsing.Literal(':') +
      pyparsing.Word(pyparsing.nums, exact=2) + pyparsing.Literal('.') +
      pyparsing.Word(pyparsing.nums, exact=6) + pyparsing.Literal('Z'))

  # A log line is defined as in the AWS ELB documentation
  _LOG_LINE = (
      _WORD.setResultsName('request_type') +
      _DATE_TIME_ISOFORMAT_STRING.setResultsName('time') +
      _WORD.setResultsName('resource_identifier') +
      _CLIENT_IP_ADDRESS_PORT.setResultsName('source_ip_port') +
      _DESTINATION_IP_ADDRESS_PORT.setResultsName('destination_ip_port') +
      _FLOAT.setResultsName('request_processing_time') +
      _FLOAT.setResultsName('destination_processing_time') +
      _FLOAT.setResultsName('response_processing_time') +
      _INTEGER.setResultsName('elb_status_code') +
      _INTEGER.setResultsName('destination_status_code') +
      _INTEGER.setResultsName('received_bytes') +
      _INTEGER.setResultsName('sent_bytes') +
      pyparsing.quotedString.setResultsName('request')
          .setParseAction(pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName('user_agent')
          .setParseAction(pyparsing.removeQuotes) +
      _WORD.setResultsName('ssl_cipher') +
      _WORD.setResultsName('ssl_protocol') +
      _WORD.setResultsName('destination_group_arn') +
      _WORD.setResultsName('trace_identifier') +
      pyparsing.quotedString.setResultsName(
          'domain_name').setParseAction(pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'chosen_cert_arn').setParseAction(pyparsing.removeQuotes) +
      _INTEGER.setResultsName('matched_rule_priority') +
      _DATE_TIME_ISOFORMAT_STRING.setResultsName('request_creation_time') +
      pyparsing.quotedString.setResultsName(
          'actions_executed').setParseAction(pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'redirect_url').setParseAction(pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'error_reason').setParseAction(pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'destination_list').setParseAction(pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'destination_status_code_list').setParseAction(
              pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'classification').setParseAction(pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'classification_reason').setParseAction(pyparsing.removeQuotes)
  )

  LINE_STRUCTURES = [('elb_accesslog', _LOG_LINE)]

  def _GetValueFromGroup(self, structure, name, key_name):
    """Retrieves a value from a Pyparsing.Group structure.

    Args:
      structure (pyparsing.ParseResults): tokens from a parsed log line.
      name (str): name of the token.
      key_name (str): key name to retrieve the value of.

    Returns:
      object: value for the specified key.
    """
    structure_value = self._GetValueFromStructure(structure, name)
    return structure_value.get(key_name)

  def _GetDateTime(self, parser_mediator, time_structure):
    """Returns a dfdatetime object from a timestamp.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
        and other components, such as storage and dfvfs.
      time_structure (str): a timestamp string of the event.

    Returns:
      TimeElements: Time elements contain separate values for year, month,
          day of month, hours, minutes and seconds.
    """
    date_time = None
    try:
      date_time = dfdatetime_time_elements.TimeElements()
      date_time.CopyFromStringISO8601(time_structure)
    except ValueError:
      parser_mediator.ProduceExtractionWarning(
          'invalid date time value: {0!s}'.format(time_structure))

    return date_time

  def ParseRecord(self, parser_mediator, key, structure):
    """Parses a log record structure and produces events.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): structure parsed from the log file.

    Raises:
      ParseError: when the structure type is unsupported.
    """
    if key != 'elb_accesslog':
      raise errors.ParseError(
          'Unable to parse record, unknown structure: {0:s}'.format(key))

    time_response_sent = structure.get('time')
    time_request_received = structure.get('request_creation_time')

    date_time_response_sent = self._GetDateTime(
        parser_mediator, time_response_sent)
    date_time_request_received = self._GetDateTime(
        parser_mediator, time_request_received)

    if date_time_request_received is None or date_time_response_sent is None:
      return

    event_data = AWSELBEventData()
    event_data.request_type = self._GetValueFromStructure(
        structure, 'request_type')
    event_data.resource_identifier = self._GetValueFromStructure(
        structure, 'resource_identifier')
    event_data.source_ip_address = self._GetValueFromGroup(
        structure, 'source_ip_port', 'source_ip_address')
    event_data.source_port = self._GetValueFromGroup(
        structure, 'source_ip_port', 'source_port')
    event_data.destination_ip_address = self._GetValueFromGroup(
        structure, 'destination_ip_port', 'destination_ip_address')
    event_data.destination_port = self._GetValueFromGroup(
        structure, 'destination_ip_port', 'destination_port')
    event_data.request_processing_time = self._GetValueFromStructure(
        structure, 'request_processing_time')
    event_data.destination_processing_time = self._GetValueFromStructure(
        structure, 'destination_processing_time')
    event_data.response_processing_time = self._GetValueFromStructure(
        structure, 'response_processing_time')
    event_data.elb_status_code = self._GetValueFromStructure(
        structure, 'elb_status_code')
    event_data.destination_status_code = self._GetValueFromStructure(
        structure, 'destination_status_code')
    event_data.received_bytes = self._GetValueFromStructure(
        structure, 'received_bytes')
    event_data.sent_bytes = self._GetValueFromStructure(
        structure, 'sent_bytes')
    event_data.request = self._GetValueFromStructure(
        structure, 'request')
    event_data.user_agent = self._GetValueFromStructure(
        structure, 'user_agent')
    event_data.ssl_cipher = self._GetValueFromStructure(
        structure, 'ssl_cipher')
    event_data.ssl_protocol = self._GetValueFromStructure(
        structure, 'ssl_protocol')
    event_data.destination_group_arn = self._GetValueFromStructure(
        structure, 'destination_group_arn')
    event_data.trace_identifier = self._GetValueFromStructure(
        structure, 'trace_identifier')
    event_data.domain_name = self._GetValueFromStructure(
        structure, 'domain_name')
    event_data.chosen_cert_arn = self._GetValueFromStructure(
        structure, 'chosen_cert_arn')
    event_data.matched_rule_priority = self._GetValueFromStructure(
        structure, 'matched_rule_priority')
    event_data.actions_executed = self._GetValueFromStructure(
        structure, 'actions_executed')
    event_data.redirect_url = self._GetValueFromStructure(
        structure, 'redirect_url')
    event_data.error_reason = self._GetValueFromStructure(
        structure, 'error_reason')
    event_data.destination_status_code_list = self._GetValueFromStructure(
        structure, 'destination_status_code_list')
    event_data.classification = self._GetValueFromStructure(
        structure, 'classification')
    event_data.classification_reason = self._GetValueFromStructure(
        structure, 'classification_reason')
    destination_list = self._GetValueFromStructure(
        structure, 'destination_list')
    event_data.destination_list = destination_list.split()

    elb_response_sent_event = time_events.DateTimeValuesEvent(
        date_time_response_sent,
        definitions.TIME_DESCRIPTION_AWS_ELB_RESPONSE_SENT)

    elb_request_received_event = time_events.DateTimeValuesEvent(
        date_time_request_received,
        definitions.TIME_DESCRIPTION_AWS_ELB_REQUEST_RECEIVED)

    parser_mediator.ProduceEventWithEventData(
        elb_response_sent_event, event_data)

    parser_mediator.ProduceEventWithEventData(
        elb_request_received_event, event_data)

  def VerifyStructure(self, parser_mediator, line):
    """Verify that this file is a valid AWS ELB access log.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      line (str): line from a text file.

    Returns:
      bool: True if the line was successfully parsed.
    """
    try:
      structure = self._LOG_LINE.parseString(line)
    except pyparsing.ParseException:
      structure = None

    return bool(structure)


manager.ParsersManager.RegisterParser(AWSELBParser)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the iOS Mobile Installation log parser."""

import unittest

from plaso.parsers import ios_sysdiag_log

from tests.parsers import test_lib


class IOSSysdiagLogParserTest(test_lib.ParserTestCase):
  """Tests for the iOS Mobile Installation log parser"""

  def testParseLog(self):
    """Tests the Parse function"""
    parser = ios_sysdiag_log.IOSSysdiagLogParser()
    storage_writer = self._ParseFile(['ios_sysdiag.log'], parser)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 28)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    events = list(storage_writer.GetEvents())

    expected_event_values = {
        'body': (
            'Ignoring plugin at /System/Library/PrivateFrameworks/'
            'AccessibilityUtilities.framework/PlugIns/com.apple.accessibility.'
            'Accessibility.HearingAidsTapToRadar.appex due to validation '
            'issue(s). See previous log messages for details.'),
        'originating_call': (
            '+[MILaunchServicesDatabaseGatherer '
            'enumeratePluginKitPluginsInBundle:updatingPluginParentID:'
            'ensurePluginsAreExecutable:installProfiles:error:enumerator:]'),
        'process_identifier': '176',
        'severity': 'err',
        'timestamp': '2021-08-11 05:51:02.000000'}

    self.CheckEventValues(storage_writer, events[7], expected_event_values)

    expected_event_values = {
        'body': (
            '47: Failed to load Info.plist from bundle at path /System/'
            'Library/PrivateFrameworks/'
            'CoreDuetContext.framework; Extra info about "/System/Library/'
            'PrivateFrameworks/'
            'CoreDuetContext.framework/Info.plist": dev=805306369 '
            'ino=1152921500312080876 mode=0100644 nlink=1 '
            'uid=0 gid=0 rdev=0 size=800 atime=1577865600.000000 '
            'mtime=1577865600.000000 '
            'ctime=1577865600.000000 birthtime=1577865600.000000 '
            'blksize=4096 blocks=0 flags=0x20 '
            'firstBytes={length = 4, bytes = 0x62706c69} ACL=<not found> '
            'extendedAttributes={\n    '
            '"com.apple.decmpfs" = {length = 817, bytes = 0x66706d63 '
            '09000000 20030000 00000000 ... 00000000 '
            '000002a6 };\n} keyCount=22 keySample={ CFBundleName DTXcode '
            'DTSDKName DTSDKBuild '
            'CFBundleDevelopmentRegion }'),
        'originating_call': '-[MIBundle _validateWithError:]',
        'process_identifier': '176',
        'severity': 'err',
        'timestamp': '2021-08-11 05:51:03.000000'}

    self.CheckEventValues(storage_writer, events[14], expected_event_values)

    expected_event_values = {
        'body': 'containermanagerd first boot cleanup complete',
        'originating_call': '_containermanagerd_init_block_invoke',
        'process_identifier': '66',
        'severity': 'notice',
        'timestamp': '2021-01-17 11:20:29.000000'}

    self.CheckEventValues(storage_writer, events[20], expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for Santa log parser."""

import unittest

from plaso.parsers import santa

from tests.parsers import test_lib


class SantaUnitTest(test_lib.ParserTestCase):
  """Tests for Santa log parser."""

  def testParseLegacyFormat(self):
    """Tests the Parse function on the legacy Santa log format."""
    parser = santa.SantaParser()
    storage_writer = self._ParseFile(['santa.log'], parser)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 208)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # The order in which parser generates events is nondeterministic hence
    # we sort the events.
    events = list(storage_writer.GetSortedEvents())

    # Execution event with quarantine URL.
    expected_event_values = {
        'date_time': '2018-08-19 03:17:55.765',
        'data_type': 'santa:execution',
        'decision': 'ALLOW',
        'process_hash': (
            '78b43a13b5b608fe1a5a590f5f3ff112ff16ece7befc29fc84347125f6b9ca78'),
        'process_path': '/Applications/Skype.app/Contents/MacOS/Skype',
        'quarantine_url': (
            'https://endpoint920510.azureedge.net/s4l/s4l/download/mac/'
            'Skype-8.28.0.41.dmg')}

    self.CheckEventValues(storage_writer, events[46], expected_event_values)

    # File operation event log
    expected_event_values = {
        'action': 'WRITE',
        'date_time': '2018-08-19 04:02:47.911',
        'data_type': 'santa:file_system_event',
        'file_path': '/Users/qwerty/newfile',
        'gid': '20',
        'group': 'staff',
        'pid': '303',
        'ppid': '1',
        'process': 'Finder',
        'process_path': (
            '/System/Library/CoreServices/Finder.app/Contents/MacOS/Finder'),
        'uid': '501',
        'user': 'qwerty'}

    self.CheckEventValues(storage_writer, events[159], expected_event_values)

    # Test a Disk mounts event.
    expected_event_values = {
        'action': 'DISKAPPEAR',
        'appearance': '2018-08-19T03:17:28.982Z',
        'bsd_name': 'disk2s1',
        'bus': 'Virtual Interface',
        'date_time': '2018-08-19 03:17:29.036',
        'data_type': 'santa:diskmount',
        'dmg_path': '/Users/qwerty/Downloads/Skype-8.28.0.41.dmg',
        'fs': 'hfs',
        'model': 'Apple Disk Image',
        'mount': '',
        'serial': '',
        'volume': 'Skype'}

    self.CheckEventValues(storage_writer, events[38], expected_event_values)

    # Test Disk event created from appearance timestamp.
    expected_event_values = {
        'date_time': '2018-08-19 03:17:28.982',
        'data_type': 'santa:diskmount',
        'timestamp_desc': 'First Connection Time'}

    self.CheckEventValues(storage_writer, events[35], expected_event_values)

  def testParseCurrentFormat(self):
    """Tests Parse function on the current Santa log format."""
    parser = santa.SantaParser()
    storage_writer = self._ParseFile(['santa2.log'], parser)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 14)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # The order in which parser generates events is nondeterministic hence
    # we sort the events.
    events = list(storage_writer.GetSortedEvents())

    # File rename operation event log
    expected_event_values = {
        'action': 'RENAME',
        'date_time': '2022-02-17 16:30:05.253',
        'data_type': 'santa:file_system_event',
        'file_path': '/private/var/db/santa/santa.log.5.gz',
        'file_new_path': '/private/var/db/santa/santa.log.6.gz',
        'pid': '26150',
        'pid_version': '2228280',
        'ppid': '1',
        'process': 'newsyslog',
        'process_path': '/usr/sbin/newsyslog',
        'uid': '0',
        'gid': '0',
        'user': 'root',
        'group': 'wheel'}

    self.CheckEventValues(storage_writer, events[3], expected_event_values)

    # Process exit log
    expected_event_values = {
        'action': 'EXIT',
        'date_time': '2022-02-09 16:47:02.893',
        'pid': '75780',
        'pid_version': '1765713',
        'ppid': '155',
        'uid': '0',
        'gid': '1'}

    self.CheckEventValues(storage_writer, events[1], expected_event_values)

    # Link file operation event log
    expected_event_values = {
        'action': 'LINK',
        'date_time': '2022-02-17 16:30:05.257',
        'data_type': 'santa:file_system_event',
        'file_path': '/private/var/db/santa/santa.log',
        'file_new_path': '/private/var/db/santa/santa.log.0',
        'pid': '26150',
        'pid_version': '2228280',
        'ppid': '1',
        'process': 'newsyslog',
        'process_path': '/usr/sbin/newsyslog',
        'uid': '0',
        'gid': '0',
        'user': 'root',
        'group': 'wheel'}

    self.CheckEventValues(storage_writer, events[4], expected_event_values)

    # Execution event log
    expected_event_values = {
        'action': 'EXEC',
        'date_time': '2022-02-17 16:31:28.414',
        'data_type': 'santa:execution',
        'decision': 'ALLOW',
        'reason': 'BINARY',
        'long_reason': 'critical system binary',
        'process_hash':
            '25977d1584525b571fc6d19dcac1a768d0555c58777876869255de312dcfcaf3',
        'certificate_hash':
            'd84db96af8c2e60ac4c851a21ec460f6f84e0235beb17d24a78712b9b021ed57',
        'certificate_common_name': 'Software Signing',
        'pid': '26364',
        'pid_version': '2228711',
        'ppid': '1',
        'uid': '0',
        'gid': '0',
        'user': 'root',
        'group': 'wheel',
        'mode': 'M',
        'process_path': '/usr/libexec/xpcproxy',
        'process_arguments': ('xpcproxy '
            'com.apple.mdworker.shared.07000000-0000-0000-0000-000000000000')}

    self.CheckEventValues(storage_writer, events[6], expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the OpenSearch output module."""

import unittest

from unittest.mock import MagicMock

from plaso.output import opensearch
from plaso.output import shared_opensearch

from tests.output import test_lib


class TestOpenSearchOutputModule(opensearch.OpenSearchOutputModule):
  """OpenSearch output module for testing."""

  def _Connect(self):
    """Connects to an OpenSearch server."""
    self._client = MagicMock()


@unittest.skipIf(
    shared_opensearch.opensearchpy is None, 'missing opensearch-py')
class OpenSearchOutputModuleTest(test_lib.OutputModuleTestCase):
  """Tests for the OpenSearch output module."""

  # pylint: disable=protected-access

  def testWriteHeader(self):
    """Tests the WriteHeader function."""
    output_mediator = self._CreateOutputMediator()
    output_module = TestOpenSearchOutputModule(output_mediator)

    self.assertIsNone(output_module._client)

    output_module.WriteHeader()

    self.assertIsNotNone(output_module._client)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains the interface for analysis plugins."""

import abc
import collections
import queue
import threading
import time

import requests

from plaso.analysis import interface
from plaso.analysis import logger
from plaso.containers import events
from plaso.lib import errors


class HashAnalysis(object):
  """Analysis information about a hash.

  Attributes:
    hash_information (object): object containing information about the hash.
    subject_hash (str): hash that was analyzed.
  """

  def __init__(self, subject_hash, hash_information):
    """Initializes analysis information about a hash.

    Args:
      subject_hash (str): hash that the hash_information relates to.
      hash_information (object): information about the hash. This object will be
          used by the GenerateLabels method in the HashTaggingAnalysisPlugin
          to tag events that relate to the hash.
    """
    self.hash_information = hash_information
    self.subject_hash = subject_hash


class HashAnalyzer(threading.Thread):
  """Interface of a hash analyzer threads.

  Attributes:
    analyses_performed (int): number of analysis batches completed by this
        analyzer.
    hashes_per_batch (int): maximum number of hashes to analyze at once.
    lookup_hash (str): name of the hash attribute to look up.
    seconds_spent_analyzing (int): number of seconds this analyzer has spent
        performing analysis (as opposed to waiting on queues, etc.)
    wait_after_analysis (int): number of seconds the analyzer will sleep for
        after analyzing a batch of hashes.
  """
  # How long to wait for new items to be added to the the input queue.
  EMPTY_QUEUE_WAIT_TIME = 4

  # List of lookup hashes supported by the analyzer.
  SUPPORTED_HASHES = []

  def __init__(
      self, hash_queue, hash_analysis_queue, hashes_per_batch=1,
      lookup_hash='sha256', wait_after_analysis=0):
    """Initializes a hash analyzer.

    Args:
      hash_queue (queue.Queue): contains hashes to be analyzed.
      hash_analysis_queue (queue.Queue): queue that the analyzer will append
          HashAnalysis objects to.
      hashes_per_batch (Optional[int]): number of hashes to analyze at once.
      lookup_hash (Optional[str]): name of the hash attribute to look up.
      wait_after_analysis (Optional[int]): number of seconds to wait after each
          batch is analyzed.
    """
    super(HashAnalyzer, self).__init__()
    self._abort = False
    self._hash_queue = hash_queue
    self._hash_analysis_queue = hash_analysis_queue

    self.analyses_performed = 0
    self.hashes_per_batch = hashes_per_batch
    self.lookup_hash = lookup_hash
    self.seconds_spent_analyzing = 0
    self.wait_after_analysis = wait_after_analysis

  def _GetHashesFromQueue(self, maximum_number_of_hashes):
    """Retrieves hashes from the queue.

    Args:
      maximum_number_of_hashes (int): maximum number of hashes to retrieve
          from the queue.

    Returns:
      list[str]: hashes.
    """
    hashes = []
    for _ in range(maximum_number_of_hashes):
      try:
        item = self._hash_queue.get_nowait()
        hashes.append(item)
      except queue.Empty:
        continue

    return hashes

  @abc.abstractmethod
  def Analyze(self, hashes):
    """Analyzes a list of hashes.

    Args:
      hashes (list[str]): list of hashes to look up.

    Returns:
      list[HashAnalysis]: list of results of analyzing the hashes.
    """

  def SetLookupHash(self, lookup_hash):
    """Sets the lookup hash to query.

    Args:
      lookup_hash (str): name of the hash attribute to look up.

    Raises:
      ValueError: if the lookup hash is not supported.
    """
    if lookup_hash not in self.SUPPORTED_HASHES:
      raise ValueError('Unsupported lookup hash: {0!s}'.format(lookup_hash))

    self.lookup_hash = lookup_hash

  # This method is part of the threading.Thread interface, hence its name does
  # not follow the style guide.
  def run(self):
    """The method called by the threading library to start the thread."""
    while not self._abort:
      hashes = self._GetHashesFromQueue(self.hashes_per_batch)
      if not hashes:
        # Wait for some more hashes to be added to the queue.
        time.sleep(self.EMPTY_QUEUE_WAIT_TIME)
        continue

      time_before_analysis = time.time()
      hash_analyses = self.Analyze(hashes)
      time_after_analysis = time.time()

      self.seconds_spent_analyzing += time_after_analysis - time_before_analysis
      self.analyses_performed += 1

      for hash_analysis in hash_analyses:
        self._hash_analysis_queue.put(hash_analysis)
        self._hash_queue.task_done()

      time.sleep(self.wait_after_analysis)

  def SignalAbort(self):
    """Instructs the hash analyzer to abort."""
    self._abort = True


class HTTPHashAnalyzer(HashAnalyzer):
  """Interface for hash analysis thread that uses HTTP(S)"""

  @abc.abstractmethod
  def Analyze(self, hashes):
    """Analyzes a list of hashes.

    Args:
      hashes (list[str]): hashes to look up.

    Returns:
      list[HashAnalysis]: analysis results.
    """

  def MakeRequestAndDecodeJSON(self, url, method, **kwargs):
    """Make a HTTP request and decode the results as JSON.

    Args:
      url (str): URL to make a request to.
      method (str): HTTP method to used to make the request. GET and POST are
          supported.
      kwargs: parameters to the requests .get() or post() methods, depending
          on the value of the method parameter.

    Returns:
      dict[str, object]: body of the HTTP response, decoded from JSON.

    Raises:
      ConnectionError: If it is not possible to connect to the given URL, or it
          the request returns a HTTP error.
      ValueError: If an invalid HTTP method is specified.
    """
    method_upper = method.upper()
    if method_upper not in ('GET', 'POST'):
      raise ValueError('Method {0:s} is not supported')

    try:
      if method_upper == 'GET':
        response = requests.get(url, **kwargs)

      elif method_upper == 'POST':
        response = requests.post(url, **kwargs)

      response.raise_for_status()

    except requests.ConnectionError as exception:
      error_string = 'Unable to connect to {0:s} with error: {1!s}'.format(
          url, exception)
      raise errors.ConnectionError(error_string)

    except requests.HTTPError as exception:
      error_string = '{0:s} returned a HTTP error: {1!s}'.format(
          url, exception)
      raise errors.ConnectionError(error_string)

    return response.json()


class HashTaggingAnalysisPlugin(interface.AnalysisPlugin):
  """An interface for plugins that tag events based on the source file hash.

  An implementation of this class should be paired with an implementation of
  the HashAnalyzer interface.
  """
  # The event data types the plugin will collect hashes from. Subclasses
  # must override this attribute.
  DATA_TYPES = []

  # The default number of seconds for the plugin to wait for analysis results
  # to be added to the hash_analysis_queue by the analyzer thread.
  DEFAULT_QUEUE_TIMEOUT = 4
  SECONDS_BETWEEN_STATUS_LOG_MESSAGES = 30

  def __init__(self, analyzer_class):
    """Initializes a hash tagging analysis plugin.

    Args:
      analyzer_class (type): a subclass of HashAnalyzer that will be
          instantiated by the plugin.
    """
    super(HashTaggingAnalysisPlugin, self).__init__()
    self._analysis_queue_timeout = self.DEFAULT_QUEUE_TIMEOUT
    self._analyzer_started = False
    self._data_stream_identifiers = set()
    self._data_streams_by_hash = collections.defaultdict(set)
    self._event_identifiers_by_data_stream = collections.defaultdict(set)
    self._hash_analysis_queue = queue.Queue()
    self._hash_queue = queue.Queue()
    self._requester_class = None
    self._time_of_last_status_log = time.time()

    self._analyzer = analyzer_class(self._hash_queue, self._hash_analysis_queue)

  def _ContinueReportCompilation(self):
    """Determines if the plugin should continue trying to compile the report.

    Returns:
      bool: True if the plugin should continue, False otherwise.
    """
    analyzer_alive = self._analyzer.is_alive()
    hash_queue_has_tasks = self._hash_queue.unfinished_tasks > 0
    analysis_queue = not self._hash_analysis_queue.empty()

    # pylint: disable=consider-using-ternary
    return (analyzer_alive and hash_queue_has_tasks) or analysis_queue

  def _EnsureAnalyzerThreadIsStarted(self):
    """Checks if the analyzer thread is running and starts it if not."""
    if not self._analyzer_started:
      self._analyzer.start()
      self._analyzer_started = True

  # pylint: disable=redundant-returns-doc
  @abc.abstractmethod
  def _GenerateLabels(self, hash_information):
    """Generates a list of strings to tag events with.

    Args:
      hash_information (bool): response from the hash tagging analyzer that
          indicates that the file hash was present or not.

    Returns:
      list[str]: list of labels to apply to event.
    """

  # TODO: Refactor to do this more elegantly, perhaps via callback.
  def _LogProgressUpdateIfReasonable(self):
    """Prints a progress update if enough time has passed."""
    next_log_time = (
        self._time_of_last_status_log +
        self.SECONDS_BETWEEN_STATUS_LOG_MESSAGES)
    current_time = time.time()
    if current_time < next_log_time:
      return

    completion_time = time.ctime(current_time + self.EstimateTimeRemaining())
    log_message = (
        '{0:s} hash analysis plugin running. {1:d} hashes in queue, '
        'estimated completion time {2:s}.'.format(
            self.NAME, self._hash_queue.qsize(), completion_time))
    logger.info(log_message)
    self._time_of_last_status_log = current_time

  def _ProcessHashAnalysis(self, analysis_mediator, hash_analysis):
    """Processes the results of the analysis of a hash.

    This method ensures that labels are generated for the hash,
    then tags all events derived from files with that hash.

    Args:
      analysis_mediator (AnalysisMediator): mediates interactions between
          analysis plugins and other components, such as storage and dfVFS.
      hash_analysis (HashAnalysis): hash analysis plugin's results for a given
          hash.
    """
    labels = self._GenerateLabels(hash_analysis.hash_information)

    try:
      data_stream_identifiers = self._data_streams_by_hash.pop(
          hash_analysis.subject_hash)
    except KeyError:
      data_stream_identifiers = []
      logger.error((
          'unable to retrieve data streams for digest hash: {0:s}').format(
              hash_analysis.subject_hash))

    for data_stream_identifier in data_stream_identifiers:
      event_identifiers = self._event_identifiers_by_data_stream.pop(
          data_stream_identifier)

      # Do no bail out earlier to maintain the state of
      # self._data_streams_by_hash and self._event_identifiers_by_data_stream.
      if not labels:
        continue

      for event_identifier in event_identifiers:
        event_tag = events.EventTag()
        event_tag.SetEventIdentifier(event_identifier)

        try:
          event_tag.AddLabels(labels)
        except (TypeError, ValueError):
          error_label = 'error_{0:s}'.format(self.NAME)
          logger.error((
              'unable to add labels: {0!s} for digest hash: {1:s} defaulting '
              'to: {2:s}').format(
                  labels, hash_analysis.subject_hash, error_label))
          labels = [error_label]
          event_tag.AddLabels(labels)

        analysis_mediator.ProduceEventTag(event_tag)

        for label in labels:
          self._analysis_counter[label] += 1

  def CompileReport(self, analysis_mediator):
    """Compiles an analysis report.

    Args:
      analysis_mediator (AnalysisMediator): mediates interactions between
          analysis plugins and other components, such as storage and dfVFS.

    Returns:
      AnalysisReport: report.
    """
    while self._ContinueReportCompilation():
      try:
        self._LogProgressUpdateIfReasonable()
        hash_analysis = self._hash_analysis_queue.get(
            timeout=self._analysis_queue_timeout)
      except queue.Empty:
        # The result queue is empty, but there could still be items that need
        # to be processed by the hash analyzer.
        continue

      self._ProcessHashAnalysis(analysis_mediator, hash_analysis)

    self._analyzer.SignalAbort()

    return super(HashTaggingAnalysisPlugin, self).CompileReport(
        analysis_mediator)

  def EstimateTimeRemaining(self):
    """Estimates how long until all hashes have been analyzed.

    Returns:
      int: estimated number of seconds until all hashes have been analyzed.
    """
    number_of_hashes = self._hash_queue.qsize()
    hashes_per_batch = self._analyzer.hashes_per_batch
    wait_time_per_batch = self._analyzer.wait_after_analysis
    analyses_performed = self._analyzer.analyses_performed

    if analyses_performed == 0:
      average_analysis_time = self._analyzer.seconds_spent_analyzing
    else:
      average_analysis_time, _ = divmod(
          self._analyzer.seconds_spent_analyzing, analyses_performed)

    batches_remaining, _ = divmod(number_of_hashes, hashes_per_batch)
    estimated_seconds_per_batch = average_analysis_time + wait_time_per_batch
    return batches_remaining * estimated_seconds_per_batch

  def ExamineEvent(
      self, analysis_mediator, event, event_data, event_data_stream):
    """Evaluates whether an event contains the right data for a hash lookup.

    Args:
      analysis_mediator (AnalysisMediator): mediates interactions between
          analysis plugins and other components, such as storage and dfVFS.
      event (EventObject): event.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.
    """
    if (not self._analyzer.lookup_hash or not event_data_stream or
        event_data.data_type not in self.DATA_TYPES):
      return

    self._EnsureAnalyzerThreadIsStarted()

    data_stream_identifier = event_data_stream.GetIdentifier()
    if data_stream_identifier not in self._data_stream_identifiers:
      self._data_stream_identifiers.add(data_stream_identifier)

      lookup_hash = '{0:s}_hash'.format(self._analyzer.lookup_hash)
      lookup_hash = getattr(event_data_stream, lookup_hash, None)

      if lookup_hash:
        self._data_streams_by_hash[lookup_hash].add(data_stream_identifier)
        self._hash_queue.put(lookup_hash)
      else:
        path_specification = getattr(event_data_stream, 'path_spec', None)
        display_name = analysis_mediator.GetDisplayNameForPathSpec(
            path_specification)
        logger.warning((
            'Lookup hash attribute: {0:s}_hash missing from event data stream: '
            '{1:s}.').format(self._analyzer.lookup_hash, display_name))

    event_identifier = event.GetIdentifier()
    self._event_identifiers_by_data_stream[data_stream_identifier].add(
        event_identifier)

  def SetLookupHash(self, lookup_hash):
    """Sets the hash to query.

    Args:
      lookup_hash (str): name of the hash attribute to look up.
    """
    self._analyzer.SetLookupHash(lookup_hash)



# -*- coding: utf-8 -*-
"""Tests for the SQLite parser plugin for iOS netusage database files."""

import unittest

from plaso.parsers.sqlite_plugins import ios_netusage

from tests.parsers.sqlite_plugins import test_lib


class IOSNetusagePluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the SQLite parser plugin for iOS netusage database files."""

  def testParseNetusageRouteRow(self):
    """Tests the ParseNetusageRouteRow method."""
    plugin = ios_netusage.IOSNetusagePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['netusage.sqlite'], plugin)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 384)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    events = list(storage_writer.GetEvents())

    expected_event_values = {
        'bytes_in': 131926734,
        'bytes_out': 59786697,
        'network_identifier': 'Carrier-18005506',
        'network_signature': '353537613832343930343563356433642D33613937333034',
        'network_type': 2,
        'timestamp': '2021-02-19 05:00:00.000000'}

    self.CheckEventValues(storage_writer, events[0], expected_event_values)

    expected_event_values = {
        'bytes_in': 34148,
        'bytes_out': 42416,
        'network_identifier': 'CcookiesDcastleR5 Guest-f8:bb:bf:8d:b9:c9',
        'network_signature': 'D9BA0E7A16AECFFC9DD5E8DC19242A49F7B2567900000000',
        'network_type': 1,
        'timestamp': '2021-02-19 05:00:00.000000'}

    self.CheckEventValues(storage_writer, events[17], expected_event_values)

  def testParseNetusageProcessRow(self):
    """Tests the ParseNetusageProcessRow method."""
    plugin = ios_netusage.IOSNetusagePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['netusage.sqlite'], plugin)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 384)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    events = list(storage_writer.GetEvents())

    expected_event_values = {
        'process_name': 'mDNSResponder',
        'wifi_in': 79188353,
        'wifi_out': 35398277,
        'wired_in': 12198756,
        'wired_out': 10841,
        'wireless_wan_in': 300997,
        'wireless_wan_out': 938811,
        'timestamp': '2021-01-17 19:23:38.212656'}

    self.CheckEventValues(storage_writer, events[50], expected_event_values)

    expected_event_values = {
        'process_name': 'imgurmobile',
        'wifi_in': 31296315,
        'wifi_out': 435462,
        'wired_in': 0,
        'wired_out': 0,
        'wireless_wan_in': 0,
        'wireless_wan_out': 0,
        'timestamp': '2021-02-19 01:06:15.397861'}

    self.CheckEventValues(storage_writer, events[376], expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for Confluence access log parser."""

import unittest

from plaso.parsers import confluence_access

from tests.parsers import test_lib


class ConfluenceAccessUnitTest(test_lib.ParserTestCase):
  """Tests for Confluence access log parser."""

  def testParsePre711(self):
    """Tests the Parse function on a pre 7.11 access log."""
    parser = confluence_access.ConfluenceAccessParser()
    storage_writer = self._ParseFile(['confluence_access.log'], parser)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 3)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # The order in which parser generates events is nondeterministic hence
    # we sort the events.
    events = list(storage_writer.GetSortedEvents())

    # 17/Jun/2021:12:57:26 +0200
    expected_event_values = {
        'data_type': 'confluence:access',
        'date_time': '2021-06-17 12:57:26',
        'http_request_referer': 'http://localhost/',
        'http_request_user_agent': (
            'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36'
            ' (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36'),
        'http_response_code': 200,
        'http_response_bytes': 7881,
        'remote_name': '192.168.192.1',
        'user_name': 'user',
        'http_version': 'HTTP/1.1',
        'process_duration': 1020,
        'http_request_method': 'GET',
        'http_request_uri': '/index.action',
        'thread_name': 'http-nio-8080-exec-6',
        'forwarded_for': None
    }

    self.CheckEventValues(storage_writer, events[0], expected_event_values)

  def testParsePost711(self):
    """Tests the Parse function on a post 7.11 access log."""
    parser = confluence_access.ConfluenceAccessParser()
    storage_writer = self._ParseFile(['confluence_access_post7.11.log'], parser)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # The order in which parser generates events is nondeterministic hence
    # we sort the events.
    events = list(storage_writer.GetSortedEvents())

    # 17/Jun/2021:12:57:26 +0200
    expected_event_values = {
        'data_type': 'confluence:access',
        'date_time': '2021-06-17 12:57:26',
        'http_request_referer': 'http://localhost/',
        'http_request_user_agent': (
            'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36'
            ' (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36'),
        'http_response_code': 200,
        'http_response_bytes': 7881,
        'remote_name': '0:0:0:0:0:0:0:1',
        'user_name': 'testuser',
        'http_version': 'HTTP/1.1',
        'process_duration': 1020,
        'http_request_method': 'GET',
        'http_request_uri': '/index.action',
        'thread_name': 'http-nio-8080-exec-6',
        'forwarded_for': '192.168.1.15'
    }

    self.CheckEventValues(storage_writer, events[0], expected_event_values)

if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*
"""Tests for the JSON-L parser plugin for Google Cloud (GCP) log files."""

import unittest

from plaso.parsers.jsonl_plugins import gcp_log

from tests.parsers.jsonl_plugins import test_lib


class GCPLogJSONLPluginTest(test_lib.JSONLPluginTestCase):
  """Tests for the JSON-L parser plugin for Google Cloud (GCP) log files."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = gcp_log.GCPLogJSONLPlugin()
    storage_writer = self._ParseJSONLFileWithPlugin(
        ['gcp_logging.jsonl'], plugin)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 9)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    events = list(storage_writer.GetEvents())

    expected_event_values = {
        'timestamp': '2021-10-19 02:57:47.339377',
        'user': 'fakeemailxyz@gmail.com'}
    self.CheckEventValues(storage_writer, events[0], expected_event_values)

    expected_event_values = {
        'timestamp': '2021-10-19 02:57:39.354769',
        'user': 'fakeemailxyz@gmail.com'}
    self.CheckEventValues(storage_writer, events[1], expected_event_values)

    expected_event_values = {
        'timestamp': '2021-10-19 02:55:51.658015',
        'user': 'fakeemailxyz@gmail.com'}
    self.CheckEventValues(storage_writer, events[2], expected_event_values)

    expected_event_values = {
        'timestamp': '2021-10-19 02:55:46.097818',
        'user': 'fakeemailxyz@gmail.com'}
    self.CheckEventValues(storage_writer, events[3], expected_event_values)

    expected_event_values = {
        'timestamp': '2021-10-19 02:43:48.064377',
        'user': 'fakeemailxyz@gmail.com'}
    self.CheckEventValues(storage_writer, events[4], expected_event_values)

    expected_event_values = {
        'timestamp': '2021-10-19 02:42:22.986298',
        'user': 'fakeemailxyz@gmail.com'}
    self.CheckEventValues(storage_writer, events[5], expected_event_values)

    expected_event_values = {
        'timestamp': '2021-10-19 02:42:13.839954',
        'user': 'fakeemailxyz@gmail.com'}
    self.CheckEventValues(storage_writer, events[6], expected_event_values)

    expected_event_values = {
        'timestamp': '2021-10-19 02:05:41.496590',
        'user': 'fakeemailxyz@gmail.com'}
    self.CheckEventValues(storage_writer, events[7], expected_event_values)

    expected_event_values = {
        'timestamp': '2021-10-19 02:04:00.272384'}
    self.CheckEventValues(storage_writer, events[8], expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for iOS sysdiagnose logd files (logd.0.log)."""

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
from plaso.parsers import manager
from plaso.parsers import text_parser


class IOSSysdiagnoseLogdData(events.EventData):
  """iOS sysdiagnose logd event data.

  Attributes:
    body (str): body of the event line.
    logger (str): name of the process that generated the event.
  """

  DATA_TYPE = 'ios:sysdiagnose:logd:line'

  def __init__(self):
    """Initializes iOS sysdiagnose logd event data."""
    super(IOSSysdiagnoseLogdData, self).__init__(data_type=self.DATA_TYPE)
    self.body = None
    self.logger = None


class IOSSysdiagnoseLogdParser(text_parser.PyparsingSingleLineTextParser):
  """Parser for iOS sysdiagnose logd files (logd.0.log)."""

  NAME = 'ios_logd'
  DATA_FORMAT = 'iOS sysdiagnose logd file'

  DATE_ELEMENTS = text_parser.PyparsingConstants.DATE_ELEMENTS
  TIME_ELEMENTS = text_parser.PyparsingConstants.TIME_ELEMENTS

  _TIMESTAMP = DATE_ELEMENTS + TIME_ELEMENTS

  _TIME_DELTA = pyparsing.Word(
      pyparsing.nums + '+' + '-', exact=5).setResultsName('time_delta')

  _LOGGER = (pyparsing.SkipTo(':').setResultsName('logger') +
             pyparsing.Suppress(': '))

  _BODY = pyparsing.SkipTo(pyparsing.LineEnd()).setResultsName('body')

  _LINE_GRAMMAR = _TIMESTAMP + _TIME_DELTA + _LOGGER + _BODY

  LINE_STRUCTURES = [('log_entry', _LINE_GRAMMAR)]

  def ParseRecord(self, parser_mediator, key, structure):
    """Parses a log record structure and produces events.

    This function takes as an input a parsed pyparsing structure
    and produces an EventObject if possible from that structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: when the structure type is unknown.
    """
    if key != 'log_entry':
      raise errors.ParseError(
          'Unable to parse record, unknown structure: {0:s}'.format(key))

    year = self._GetValueFromStructure(structure, 'year')
    month = self._GetValueFromStructure(structure, 'month')
    day = self._GetValueFromStructure(structure, 'day_of_month')
    hours = self._GetValueFromStructure(structure, 'hours')
    minutes = self._GetValueFromStructure(structure, 'minutes')
    seconds = self._GetValueFromStructure(structure, 'seconds')

    time_delta = self._GetValueFromStructure(structure, 'time_delta')

    event_data = IOSSysdiagnoseLogdData()
    event_data.body = self._GetValueFromStructure(structure, 'body')
    event_data.logger = self._GetValueFromStructure(structure, 'logger')

    # dfDateTime takes the time zone offset as number of minutes relative from
    # UTC. So for Easter Standard Time (EST), which is UTC-5:00 the sign needs
    # to be converted, to +300 minutes.
    try:
      time_delta_hours = int(time_delta[:3], 10)
      time_delta_minutes = int(time_delta[3:], 10)
    except (TypeError, ValueError):
      parser_mediator.ProduceExtractionWarning('unsupported date time value')
      return

    time_zone_offset = (time_delta_hours * -60) + time_delta_minutes

    try:
      date_time = dfdatetime_time_elements.TimeElements(
          time_elements_tuple=(year, month, day, hours, minutes, seconds),
          time_zone_offset=time_zone_offset)
    except (TypeError, ValueError):
      parser_mediator.ProduceExtractionWarning('unsupported date time value')
      return

    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_MODIFICATION)

    parser_mediator.ProduceEventWithEventData(event, event_data)

  def VerifyStructure(self, parser_mediator, line):
    """Verifies that this is an iOS sysdiagnose logd file.

    Args:
      parser_mediator (ParserMediator): mediates interactions between
          parsers and other components, such as storage and dfVFS.
      line (str): one line from the text file.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    match_generator = self._LINE_GRAMMAR.scanString(line, maxMatches=1)
    return bool(list(match_generator))


manager.ParsersManager.RegisterParser(IOSSysdiagnoseLogdParser)



# -*- coding: utf-8 -*-
"""Parser related functions and classes for testing."""

from dfdatetime import posix_time as dfdatetime_posix_time

from dfvfs.file_io import fake_file_io
from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.path import fake_path_spec
from dfvfs.resolver import context as dfvfs_context
from dfvfs.resolver import resolver as path_spec_resolver

from plaso.containers import events
from plaso.engine import knowledge_base
from plaso.parsers import interface
from plaso.parsers import mediator as parsers_mediator
from plaso.storage.fake import writer as fake_writer

from tests import test_lib as shared_test_lib


class ParserTestCase(shared_test_lib.BaseTestCase):
  """Parser test case."""

  def _CreateFileObject(self, filename, data):
    """Creates a file-like object.

    Args:
      filename (str): name of the file.
      data (bytes): data of the file.

    Returns:
      dfvfs.FakeFile: file-like object.
    """
    resolver_context = dfvfs_context.Context()

    location = '/{0:s}'.format(filename)
    test_path_spec = fake_path_spec.FakePathSpec(location=location)
    file_object = fake_file_io.FakeFile(resolver_context, test_path_spec, data)
    file_object.Open()

    return file_object

  def _CreateParserMediator(
      self, storage_writer, collection_filters_helper=None,
      file_entry=None, knowledge_base_values=None, parser_chain=None,
      timezone='UTC'):
    """Creates a parser mediator.

    Args:
      storage_writer (StorageWriter): storage writer.
      collection_filters_helper (Optional[CollectionFiltersHelper]): collection
          filters helper.
      file_entry (Optional[dfvfs.FileEntry]): file entry object being parsed.
      knowledge_base_values (Optional[dict]): knowledge base values.
      parser_chain (Optional[str]): parsing chain up to this point.
      timezone (Optional[str]): timezone.

    Returns:
      ParserMediator: parser mediator.
    """
    knowledge_base_object = knowledge_base.KnowledgeBase()
    if knowledge_base_values:
      for identifier, value in knowledge_base_values.items():
        if identifier == 'codepage':
          knowledge_base_object.SetCodepage(value)
        else:
          knowledge_base_object.SetValue(identifier, value)

    knowledge_base_object.SetTimeZone(timezone)

    parser_mediator = parsers_mediator.ParserMediator(
        knowledge_base_object,
        collection_filters_helper=collection_filters_helper)
    parser_mediator.SetStorageWriter(storage_writer)

    if file_entry:
      parser_mediator.SetFileEntry(file_entry)

    if parser_chain:
      parser_mediator.parser_chain = parser_chain

    return parser_mediator

  def _CreateStorageWriter(self):
    """Creates a storage writer object.

    Returns:
      FakeStorageWriter: storage writer.
    """
    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()
    return storage_writer

  def _GetEventDataOfEvent(self, storage_writer, event):
    """Retrieves the event data of an event.

    Args:
      storage_writer (FakeStorageWriter): storage writer.
      event (EventObject): event.

    Return:
      EventData: event data corresponding to the event.
    """
    event_data_identifier = event.GetEventDataIdentifier()
    return storage_writer.GetAttributeContainerByIdentifier(
        events.EventData.CONTAINER_TYPE, event_data_identifier)

  def _ParseFile(
      self, path_segments, parser, collection_filters_helper=None,
      knowledge_base_values=None, timezone='UTC'):
    """Parses a file with a parser and writes results to a storage writer.

    Args:
      path_segments (list[str]): path segments inside the test data directory.
      parser (BaseParser): parser.
      collection_filters_helper (Optional[CollectionFiltersHelper]): collection
          filters helper.
      knowledge_base_values (Optional[dict]): knowledge base values.
      timezone (Optional[str]): time zone.

    Returns:
      FakeStorageWriter: storage writer.

    Raises:
      SkipTest: if the path inside the test data directory does not exist and
          the test should be skipped.
    """
    test_file_path = self._GetTestFilePath(path_segments)
    self._SkipIfPathNotExists(test_file_path)

    path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    return self._ParseFileByPathSpec(
        path_spec, parser, collection_filters_helper=collection_filters_helper,
        knowledge_base_values=knowledge_base_values, timezone=timezone)

  def _ParseFileByPathSpec(
      self, path_spec, parser, collection_filters_helper=None,
      knowledge_base_values=None, timezone='UTC'):
    """Parses a file with a parser and writes results to a storage writer.

    Args:
      path_spec (dfvfs.PathSpec): path specification.
      parser (BaseParser): parser.
      collection_filters_helper (Optional[CollectionFiltersHelper]): collection
          filters helper.
      knowledge_base_values (Optional[dict]): knowledge base values.
      timezone (Optional[str]): time zone.

    Returns:
      FakeStorageWriter: storage writer.
    """
    storage_writer = self._CreateStorageWriter()
    file_entry = path_spec_resolver.Resolver.OpenFileEntry(path_spec)
    parser_mediator = self._CreateParserMediator(
        storage_writer, collection_filters_helper=collection_filters_helper,
        file_entry=file_entry, knowledge_base_values=knowledge_base_values,
        timezone=timezone)

    if isinstance(parser, interface.FileEntryParser):
      parser.Parse(parser_mediator)

    elif isinstance(parser, interface.FileObjectParser):
      file_object = file_entry.GetFileObject()
      parser.Parse(parser_mediator, file_object)

    else:
      self.fail('Got unsupported parser type: {0!s}'.format(type(parser)))

    return storage_writer

  def CheckEventValues(self, storage_writer, event, expected_event_values):
    """Asserts that an event and its event data matches the expected values.

    Args:
      storage_writer (StorageWriter): storage writer.
      event (EventObject): event to check.
      expected_event_values (dict[str, list[str]): expected values of the event
          and event data attribute values per name.
    """
    event_data = None
    for name, expected_value in expected_event_values.items():
      if name == 'timestamp' and isinstance(expected_value, str):
        posix_time = dfdatetime_posix_time.PosixTimeInMicroseconds(
            timestamp=event.timestamp)
        value = posix_time.CopyToDateTimeString()

      elif name in ('date_time', 'timestamp', 'timestamp_desc'):
        value = getattr(event, name, None)

      else:
        if not event_data:
          event_data = self._GetEventDataOfEvent(storage_writer, event)

        value = getattr(event_data, name, None)

      if name == 'date_time' and value and isinstance(expected_value, str):
        value = value.CopyToDateTimeString()

      error_message = (
          'event value: "{0:s}" does not match expected value').format(name)
      self.assertEqual(value, expected_value, error_message)

  def CheckTimestamp(self, timestamp, expected_date_time):
    """Asserts that a timestamp value matches the expected date and time.

    Args:
      timestamp (int): timestamp, which contains the number of microseconds
          since January 1, 1970, 00:00:00 UTC.
      expected_date_time (str): expected date and time in UTC, formatted as:
          YYYY-MM-DD hh:mm:ss.######
    """
    posix_time = dfdatetime_posix_time.PosixTimeInMicroseconds(
        timestamp=timestamp)
    date_time = posix_time.CopyToDateTimeString()
    self.assertEqual(date_time, expected_date_time)



# -*- coding: utf-8 -*-
"""JSON-L parser plugin for Docker layer configuration files."""

from dfdatetime import semantic_time as dfdatetime_semantic_time
from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import jsonl_parser
from plaso.parsers.jsonl_plugins import interface


class DockerLayerConfigurationEventData(events.EventData):
  """Docker layer configuration event data.

  Attributes:
    command: the command used which made Docker create a new layer.
    layer_identifier: the identifier of the current Docker layer (SHA-1).
  """

  DATA_TYPE = 'docker:layer:configuration'

  def __init__(self):
    """Initializes event data."""
    super(DockerLayerConfigurationEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.command = None
    self.layer_identifier = None


class DockerLayerConfigurationJSONLPlugin(interface.JSONLPlugin):
  """JSON-L parser plugin for Docker layer configuration files.

  This parser handles per Docker layer configuration files stored in:
  DOCKER_DIR/graph/<layer_identifier>/json
  """

  NAME = 'docker_layer_config'
  DATA_FORMAT = 'Docker layer configuration file'

  def _GetLayerIdentifierFromPath(self, parser_mediator):
    """Extracts a layer (or graph) identifier from a path.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.

    Returns:
      str: layer identifier.
    """
    file_entry = parser_mediator.GetFileEntry()
    file_system = file_entry.GetFileSystem()

    path_segments = file_system.SplitPath(file_entry.path_spec.location)
    # TODO: validate format of layer identifier.
    return path_segments[-2]

  def _ParseRecord(self, parser_mediator, json_dict):
    """Parses a Docker layer configuration record.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      json_dict (dict): JSON dictionary of the configuration record.
    """
    created = self._GetJSONValue(json_dict, 'created')
    if created:
      container_configuration = self._GetJSONValue(
          json_dict, 'container_config', default_value={})
      commands = self._GetJSONValue(
          container_configuration, 'Cmd', default_value=[])

      event_data = DockerLayerConfigurationEventData()
      event_data.command = ' '.join([
          command.replace('\t', '').strip() for command in commands])
      event_data.layer_identifier = self._GetLayerIdentifierFromPath(
          parser_mediator)

      try:
        date_time = dfdatetime_time_elements.TimeElementsInMicroseconds()
        date_time.CopyFromStringISO8601(created)
      except ValueError as exception:
        parser_mediator.ProduceExtractionWarning((
            'Unable to parse created time string: {0:s} with error: '
            '{1!s}').format(created, exception))
        date_time = dfdatetime_semantic_time.InvalidTime()

      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_ADDED)
      parser_mediator.ProduceEventWithEventData(event, event_data)

  def CheckRequiredFormat(self, json_dict):
    """Check if the record has the minimal structure required by the plugin.

    Args:
      json_dict (dict): JSON dictionary of the configuration record.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    container_configuration = json_dict.get('container_config') or None
    docker_version = json_dict.get('docker_version') or None

    if None in (container_configuration, docker_version):
      return False

    return True


jsonl_parser.JSONLParser.RegisterPlugin(DockerLayerConfigurationJSONLPlugin)



# -*- coding: utf-8 -*-
"""Storage attribute container identifier objects."""

from plaso.containers import interface as containers_interface


class FakeIdentifier(containers_interface.AttributeContainerIdentifier):
  """Fake attribute container identifier intended for testing.

  Attributes:
    sequence_number (int): sequence number of the attribute container.
  """

  def __init__(self, sequence_number):
    """Initializes a fake attribute container identifier.

    Args:
      sequence_number (int): sequence number of the attribute container.
    """
    super(FakeIdentifier, self).__init__()
    self.sequence_number = sequence_number

  def CopyToString(self):
    """Copies the identifier to a string representation.

    Returns:
      str: unique identifier or None.
    """
    if self.sequence_number is None:
      return None

    return '{0:d}'.format(self.sequence_number)


class RedisKeyIdentifier(containers_interface.AttributeContainerIdentifier):
  """Redis key attribute container identifier.

  Attributes:
    name (str): name of the attribute container.
    sequence_number (int): sequence number of the attribute container.
  """

  def __init__(self, name, sequence_number):
    """"Initializes a Redis key identifier.

    Args:
      name (str): name of the table.
      sequence_number (int): sequence number of the attribute container.
    """
    super(RedisKeyIdentifier, self).__init__()
    self.name = name
    self.sequence_number = sequence_number

  def CopyToString(self):
    """Copies the identifier to a string representation.

    Returns:
      str: unique identifier or None.
    """
    if self.name is not None and self.sequence_number is not None:
      return '{0:s}.{1:d}'.format(self.name, self.sequence_number)

    return None


class SQLTableIdentifier(containers_interface.AttributeContainerIdentifier):
  """SQL table attribute container identifier.

  The identifier is used to uniquely identify attribute containers. Where
  for example an attribute container is stored as a JSON serialized data in
  a SQLite database file.

  Attributes:
    name (str): name of the table (attribute container).
    sequence_number (int): sequence number of the attribute container.
  """

  def __init__(self, name, sequence_number):
    """Initializes a SQL table attribute container identifier.

    Args:
      name (str): name of the table (attribute container).
      sequence_number (int): sequence number of the attribute container.
    """
    super(SQLTableIdentifier, self).__init__()
    self.name = name
    self.sequence_number = sequence_number

  @property
  def row_identifier(self):
    """int: unique identifier of the row in the table."""
    return self.sequence_number

  def CopyToString(self):
    """Copies the identifier to a string representation.

    Returns:
      str: unique identifier or None.
    """
    if self.name is not None and self.sequence_number is not None:
      return '{0:s}.{1:d}'.format(self.name, self.sequence_number)

    return None



# -*- coding: utf-8 -*-
"""An output module that saves events to OpenSearch."""

from plaso.output import manager
from plaso.output import shared_opensearch


class OpenSearchOutputModule(shared_opensearch.SharedOpenSearchOutputModule):
  """Output module for OpenSearch."""

  NAME = 'opensearch'
  DESCRIPTION = 'Saves the events into an OpenSearch database.'

  MAPPINGS_FILENAME = 'opensearch.mappings'

  def WriteHeader(self):
    """Connects to the OpenSearch server and creates the index."""
    self._Connect()

    self._CreateIndexIfNotExists(self._index_name, self._mappings)


manager.OutputManager.RegisterOutput(
    OpenSearchOutputModule, disabled=shared_opensearch.opensearchpy is None)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for Fish History file parser."""

import unittest

from plaso.parsers import fish_history

from tests.parsers import test_lib


class FishHistoryTest(test_lib.ParserTestCase):
  """Tests for Fish History file parser."""

  def testParseFile(self):
    """Test parsing of a Fish History file."""
    parser = fish_history.FishHistoryParser()
    storage_writer = self._ParseFile(['fish_history'], parser)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 10)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    events = list(storage_writer.GetEvents())

    expected_event_values = {
        'command': 'll', 'date_time': '2021-04-29 22:53:00'}
    self.CheckEventValues(storage_writer, events[0], expected_event_values)

    expected_event_values = {
        'command': 'la', 'date_time': '2021-04-29 22:53:02'}
    self.CheckEventValues(storage_writer, events[1], expected_event_values)

    expected_event_values = {
        'command': 'mkdir test', 'date_time': '2021-04-29 22:53:24'}
    self.CheckEventValues(storage_writer, events[2], expected_event_values)

    expected_event_values = {
        'command': 'cp *.txt test', 'date_time': '2021-04-29 22:53:32'}
    self.CheckEventValues(storage_writer, events[3], expected_event_values)

    expected_event_values = {
        'command': 'cd test', 'date_time': '2021-04-29 22:53:37'}
    self.CheckEventValues(storage_writer, events[4], expected_event_values)

    expected_event_values = {
        'command': 'rm -rf test', 'date_time': '2021-04-29 22:54:18'}
    self.CheckEventValues(storage_writer, events[5], expected_event_values)

    expected_event_values = {
        'command': 'clear', 'date_time': '2021-05-03 08:20:14'}
    self.CheckEventValues(storage_writer, events[6], expected_event_values)

    expected_event_values = {
        'command': 'pwd', 'date_time': '2021-05-03 19:08:19'}
    self.CheckEventValues(storage_writer, events[7], expected_event_values)

    expected_event_values = {
        'command': 'git clone git@github.com:log2timeline/plaso.git',
        'date_time': '2021-07-11 02:58:01'}
    self.CheckEventValues(storage_writer, events[8], expected_event_values)

    expected_event_values = {
        'command': 'cd plaso', 'date_time': '2021-07-11 02:58:35'}
    self.CheckEventValues(storage_writer, events[9], expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Output related functions and classes for testing."""

from plaso.engine import knowledge_base
from plaso.output import mediator

from tests import test_lib as shared_test_lib


class OutputModuleTestCase(shared_test_lib.BaseTestCase):
  """The unit test case for a output module."""

  def _CreateOutputMediator(self, dynamic_time=True):
    """Creates a test output mediator.

    Args:
      dynamic_time (Optional[bool]): True if date and time values should be
          represented in their granularity or semantically.

    Returns:
      OutputMediator: output mediator.
    """
    knowledge_base_object = knowledge_base.KnowledgeBase()

    return mediator.OutputMediator(
        knowledge_base_object, data_location=shared_test_lib.TEST_DATA_PATH,
        dynamic_time=dynamic_time)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the iOS sysdiagnose logd files (logd.0.log) parser."""

import unittest

from plaso.parsers import ios_logd

from tests.parsers import test_lib


class IOSSysdiagnoseLogdUnitTest(test_lib.ParserTestCase):
  """Tests for the iOS sysdiagnose logd files (logd.0.log) parser."""

  def testParseLog(self):
    """Tests the Parse function."""
    parser = ios_logd.IOSSysdiagnoseLogdParser()
    storage_writer = self._ParseFile(['logd.0.log'], parser)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 76)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    events = list(storage_writer.GetEvents())

    expected_event_values = {
        'body': 'libtrace_kic=1',
        'logger': 'logd[29]',
        'timestamp': '2021-08-10 22:50:23.000000'}

    self.CheckEventValues(storage_writer, events[2], expected_event_values)

    expected_event_values = {
        'body': 'Harvesting: 9227BC73-2AF5-318F-AA62-DCBE2219DBC8',
        'logger': 'logd_helper[90]',
        'timestamp': '2021-10-13 04:41:02.000000'}

    self.CheckEventValues(storage_writer, events[29], expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Tests for the JSON-L parser plugin for Microsoft audit log files."""

import unittest

from plaso.parsers.jsonl_plugins import microsoft_audit_log

from tests.parsers.jsonl_plugins import test_lib


class MicrosoftAuditLogJSONLPluginTest(test_lib.JSONLPluginTestCase):
  """Tests for the JSON-L parser plugin for Microsoft 365 audit log files."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = microsoft_audit_log.MicrosoftAuditLogJSONLPlugin()
    storage_writer = self._ParseJSONLFileWithPlugin(
        ['microsoft_audit_log.jsonl'], plugin)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    events = list(storage_writer.GetEvents())

    expected_event_values = {
        'audit_record_identifier': 'c9d2d808-0efe-48cb-eaec-08da3028eb80',
        'operation_name': 'Install-DefaultSharingPolicy',
        'organization_identifier': '5a0f38c6-710b-4503-92c0-3a9f6e00f726',
        'record_type': 1,
        'result_status': 'True',
        'user_key': 'NT AUTHORITY\\SYSTEM (Microsoft.Exchange.Servicehost)',
        'user_type': 3,
        'workload': 'Exchange',
        'object_identifier':
            'sst5f.onmicrosoft.com\\952cae95-808b-4aa7-b783-a9151be9a05a',
        'user_identifier':
            'NT AUTHORITY\\SYSTEM (Microsoft.Exchange.Servicehost)',
        'timestamp': '2022-05-07 12:55:53.000000'}
    self.CheckEventValues(storage_writer, events[0], expected_event_values)

    expected_event_values = {
        'audit_record_identifier': 'cd710ce3-52fa-4c70-aec8-08da3028fa73',
        'operation_name': 'Set-MailboxPlan',
        'organization_identifier': '5a0f38c6-710b-4503-92c0-3a9f6e00f726',
        'record_type': 1,
        'result_status': 'True',
        'user_key': 'NT AUTHORITY\\SYSTEM (Microsoft.Exchange.Servicehost)',
        'user_type': 3,
        'workload': 'Exchange',
        'user_identifier':
            'NT AUTHORITY\\SYSTEM (Microsoft.Exchange.Servicehost)',
        'timestamp': '2022-05-07 12:56:18.000000'}
    self.CheckEventValues(storage_writer, events[1], expected_event_values)

    expected_event_values = {
        'audit_record_identifier': 'cc0bb5b2-b540-4640-5e1c-08da3028fe4f',
        'operation_name': 'Set-AdminAuditLogConfig',
        'organization_identifier': '5a0f38c6-710b-4503-92c0-3a9f6e00f726',
        'record_type': 1,
        'result_status': 'True',
        'user_key': 'NT AUTHORITY\\SYSTEM (Microsoft.Exchange.Servicehost)',
        'user_type': 3,
        'workload': 'Exchange',
        'user_identifier':
            'NT AUTHORITY\\SYSTEM (Microsoft.Exchange.Servicehost)',
        'timestamp':'2022-05-07 12:56:24.000000'}
    self.CheckEventValues(storage_writer, events[2], expected_event_values)

    expected_event_values = {
    'audit_record_identifier': '0a454a7b-fbac-4329-a20c-72bad3bc5000',
    'operation_name': 'UserLoggedIn',
    'organization_identifier': '5a0f38c6-710b-4503-92c0-3a9f6e00f726',
    'record_type': 15,
    'result_status': 'Success',
    'user_key': '55425677-a6b7-4df1-8068-709eb4162d42',
    'user_type': 0,
    'workload': 'AzureActiveDirectory',
    'client_ip': '0.0.0.0',
    'object_identifier': '00000003-0000-0000-c000-000000000000',
    'user_identifier': 'piet@sst5f.onmicrosoft.com',
    'timestamp': '2022-05-08 15:13:41.000000'}
    self.CheckEventValues(storage_writer, events[3], expected_event_values)


if __name__ == '__main__':
  unittest.main()
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the JSON-L parser plugin for Docker container log files."""

import unittest

from plaso.parsers.jsonl_plugins import docker_container_log

from tests.parsers.jsonl_plugins import test_lib


class DockerContainerLogJSONLPluginTest(test_lib.JSONLPluginTestCase):
  """Tests for the JSON-L parser plugin for Docker container log files."""

  def testProcess(self):
    """Tests the Process function."""
    container_identifier = (
        'e7d0b7ea5ccf08366e2b0c8afa2318674e8aefe802315378125d2bb83fe3110c')
    path_segments = [
        'docker', 'containers', container_identifier, 'container-json.log']

    plugin = docker_container_log.DockerContainerLogJSONLPlugin()
    storage_writer = self._ParseJSONLFileWithPlugin(path_segments, plugin)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 10)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    events = list(storage_writer.GetEvents())

    expected_event_values = {
        'container_identifier': container_identifier,
        'data_type': 'docker:container:log:entry',
        'log_line': (
            '\x1b]0;root@e7d0b7ea5ccf: '
            '/home/plaso\x07root@e7d0b7ea5ccf:/home/plaso# ls\r\n'),
        'log_source': 'stdout',
        'timestamp': '2016-01-07 16:49:10.000000'}
    self.CheckEventValues(storage_writer, events[0], expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the fake (in-memory only) store."""

import unittest

from plaso.containers import events
from plaso.storage.fake import fake_store

from tests.storage import test_lib
from tests.containers import test_lib as containers_test_lib


class FakeStoreTest(test_lib.StorageTestCase):
  """Tests for the fake (in-memory only) store."""

  # pylint: disable=protected-access

  # TODO: add tests for _RaiseIfNotReadable
  # TODO: add tests for _RaiseIfNotWritable

  def testWriteExistingAttributeContainer(self):
    """Tests the _WriteExistingAttributeContainer function."""
    event_data_stream = events.EventDataStream()

    test_store = fake_store.FakeStore()
    test_store.Open()

    try:
      number_of_containers = test_store.GetNumberOfAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 0)

      with self.assertRaises(IOError):
        test_store._WriteExistingAttributeContainer(event_data_stream)

      test_store._WriteNewAttributeContainer(event_data_stream)

      number_of_containers = test_store.GetNumberOfAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 1)

      test_store._WriteExistingAttributeContainer(event_data_stream)

      number_of_containers = test_store.GetNumberOfAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 1)

    finally:
      test_store.Close()

  def testWriteNewAttributeContainer(self):
    """Tests the _WriteNewAttributeContainer function."""
    event_data_stream = events.EventDataStream()

    test_store = fake_store.FakeStore()
    test_store.Open()

    try:
      number_of_containers = test_store.GetNumberOfAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 0)

      test_store._WriteNewAttributeContainer(event_data_stream)

      number_of_containers = test_store.GetNumberOfAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 1)

    finally:
      test_store.Close()

  def testAddAttributeContainer(self):
    """Tests the AddAttributeContainer function."""
    event_data_stream = events.EventDataStream()

    test_store = fake_store.FakeStore()
    test_store.Open()

    try:
      number_of_containers = test_store.GetNumberOfAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 0)

      test_store.AddAttributeContainer(event_data_stream)

      number_of_containers = test_store.GetNumberOfAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 1)

    finally:
      test_store.Close()

    with self.assertRaises(IOError):
      test_store.AddAttributeContainer(event_data_stream)

  def testGetAttributeContainerByIdentifier(self):
    """Tests the GetAttributeContainerByIdentifier function."""
    event_data_stream = events.EventDataStream()

    test_store = fake_store.FakeStore()
    test_store.Open()

    try:
      test_store.AddAttributeContainer(event_data_stream)
      identifier = event_data_stream.GetIdentifier()

      container = test_store.GetAttributeContainerByIdentifier(
          event_data_stream.CONTAINER_TYPE, identifier)
      self.assertIsNotNone(container)

      identifier.sequence_number = 99

      container = test_store.GetAttributeContainerByIdentifier(
          event_data_stream.CONTAINER_TYPE, identifier)
      self.assertIsNone(container)

    finally:
      test_store.Close()

  def testGetAttributeContainerByIndex(self):
    """Tests the GetAttributeContainerByIndex function."""
    event_data_stream = events.EventDataStream()

    test_store = fake_store.FakeStore()
    test_store.Open()

    try:
      container = test_store.GetAttributeContainerByIndex(
          event_data_stream.CONTAINER_TYPE, 0)
      self.assertIsNone(container)

      test_store.AddAttributeContainer(event_data_stream)

      container = test_store.GetAttributeContainerByIndex(
          event_data_stream.CONTAINER_TYPE, 0)
      self.assertIsNotNone(container)

    finally:
      test_store.Close()

  def testGetAttributeContainers(self):
    """Tests the GetAttributeContainers function."""
    event_data_stream = events.EventDataStream()
    event_data_stream.md5_hash = '8f0bf95a7959baad9666b21a7feed79d'

    test_store = fake_store.FakeStore()
    test_store.Open()

    try:
      containers = list(test_store.GetAttributeContainers(
          event_data_stream.CONTAINER_TYPE))
      self.assertEqual(len(containers), 0)

      test_store.AddAttributeContainer(event_data_stream)

      containers = list(test_store.GetAttributeContainers(
          event_data_stream.CONTAINER_TYPE))
      self.assertEqual(len(containers), 1)

      filter_expression = 'md5_hash == "8f0bf95a7959baad9666b21a7feed79d"'
      containers = list(test_store.GetAttributeContainers(
          event_data_stream.CONTAINER_TYPE,
          filter_expression=filter_expression))
      self.assertEqual(len(containers), 1)

      filter_expression = 'md5_hash != "8f0bf95a7959baad9666b21a7feed79d"'
      containers = list(test_store.GetAttributeContainers(
          event_data_stream.CONTAINER_TYPE,
          filter_expression=filter_expression))
      self.assertEqual(len(containers), 0)

    finally:
      test_store.Close()

  def testGetNumberOfAttributeContainers(self):
    """Tests the GetNumberOfAttributeContainers function."""
    event_data_stream = events.EventDataStream()

    test_store = fake_store.FakeStore()
    test_store.Open()

    try:
      number_of_containers = test_store.GetNumberOfAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 0)

      test_store.AddAttributeContainer(event_data_stream)

      number_of_containers = test_store.GetNumberOfAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 1)

    finally:
      test_store.Close()

  def testGetSortedEvents(self):
    """Tests the GetSortedEvents function."""
    test_store = fake_store.FakeStore()
    test_store.Open()

    try:
      for event, event_data, event_data_stream in (
          containers_test_lib.CreateEventsFromValues(self._TEST_EVENTS)):
        test_store.AddAttributeContainer(event_data_stream)

        event_data.SetEventDataStreamIdentifier(
            event_data_stream.GetIdentifier())
        test_store.AddAttributeContainer(event_data)

        event.SetEventDataIdentifier(event_data.GetIdentifier())
        test_store.AddAttributeContainer(event)

      test_events = list(test_store.GetSortedEvents())
      self.assertEqual(len(test_events), 4)

    finally:
      test_store.Close()

    # TODO: add test with time range.

  def testHasAttributeContainers(self):
    """Tests the HasAttributeContainers function."""
    event_data_stream = events.EventDataStream()

    test_store = fake_store.FakeStore()
    test_store.Open()

    try:
      result = test_store.HasAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertFalse(result)

      test_store.AddAttributeContainer(event_data_stream)

      result = test_store.HasAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertTrue(result)

    finally:
      test_store.Close()

  def testOpenClose(self):
    """Tests the Open and Close functions."""
    test_store = fake_store.FakeStore()
    test_store.Open()
    test_store.Close()

    test_store.Open()
    test_store.Close()

    test_store = fake_store.FakeStore()
    test_store.Open()
    test_store.Close()

    test_store.Open()

    with self.assertRaises(IOError):
      test_store.Open()

    test_store.Close()

    with self.assertRaises(IOError):
      test_store.Close()

  def testUpdateAttributeContainer(self):
    """Tests the UpdateAttributeContainer function."""
    event_data_stream = events.EventDataStream()

    test_store = fake_store.FakeStore()
    test_store.Open()

    try:
      number_of_containers = test_store.GetNumberOfAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 0)

      with self.assertRaises(IOError):
        test_store.UpdateAttributeContainer(event_data_stream)

      test_store.AddAttributeContainer(event_data_stream)

      number_of_containers = test_store.GetNumberOfAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 1)

      test_store.UpdateAttributeContainer(event_data_stream)

      number_of_containers = test_store.GetNumberOfAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 1)

    finally:
      test_store.Close()


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""SQLite parser plugin for Dropbox sync_history database files."""

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class DropboxSyncHistoryEventData(events.EventData):
  """Dropbox Sync History Database event data.

  Attributes:
    event_type (str): the event type
    file_event_type (str): the file event type
    direction (str): the source of the synchronisation event
    file_identifier (str): the Dropbox identifier of the file.
    local_path (str): the local path of the file.
  """

  DATA_TYPE = 'dropbox:sync_history:entry'

  def __init__(self):
    """Initializes event data."""
    super(DropboxSyncHistoryEventData, self).__init__(data_type=self.DATA_TYPE)
    self.event_type = None
    self.file_event_type = None
    self.direction = None
    self.file_identifier = None
    self.local_path = None


class DropboxSyncDatabasePlugin(interface.SQLitePlugin):
  """SQLite parser plugin for Dropbox sync_history.db database files.

  The Linux sync_history.db database is typically stored in:
  $HOME/.dropbox/instance1/sync_history.db

  The Windows 10 sync_history.db database is typically stored in:
  $HOME/AppData/Local/Dropbox/instance1/sync_history.db
  """

  NAME = 'dropbox'
  DATA_FORMAT = 'Dropbox sync history database (sync_history.db) file'

  REQUIRED_STRUCTURE = {
      'sync_history': frozenset(['timestamp', 'event_type', 'file_event_type',
      'direction', 'file_id', 'local_path'])}

  QUERIES = [
      ('SELECT timestamp, event_type, file_event_type, direction, file_id, '
       'local_path FROM sync_history;',
       'ParseSyncHistoryRow')]

  SCHEMAS = [{
      'sync_history': (
          'CREATE TABLE sync_history (event_type TEXT NOT NULL, file_event_type'
          ' TEXT, direction TEXT, file_id TEXT, local_path TEXT, timestamp '
          'INTEGER NOT NULL, other_user INTEGER')}]

  def ParseSyncHistoryRow(self, parser_mediator, query, row, **unused_kwargs):
    """Parses a sync_history row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      query (str): query that created the row.
      row (sqlite3.Row): row.
    """
    query_hash = hash(query)

    event_data = DropboxSyncHistoryEventData()
    event_data.event_type = self._GetRowValue(query_hash, row, 'event_type')
    event_data.file_event_type = self._GetRowValue(
        query_hash, row, 'file_event_type')
    event_data.direction = self._GetRowValue(query_hash, row, 'direction')
    event_data.file_identifier = self._GetRowValue(query_hash, row, 'file_id')
    event_data.local_path = self._GetRowValue(query_hash, row, 'local_path')

    timestamp = self._GetRowValue(query_hash, row, 'timestamp')
    date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_RECORDED)
    parser_mediator.ProduceEventWithEventData(event, event_data)


sqlite.SQLiteParser.RegisterPlugin(DropboxSyncDatabasePlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the iOS lockdown daemon log files parser."""

import unittest

from plaso.parsers import ios_lockdownd

from tests.parsers import test_lib


class IOSLockdownLogUnitTest(test_lib.ParserTestCase):
  """Tests for the iOS lockdown daemon log files parser."""

  def testParseLog(self):
    """Tests the Parse function."""
    parser = ios_lockdownd.IOSLockdownParser()
    storage_writer = self._ParseFile(['ios_lockdownd.log'], parser)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 153)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    events = list(storage_writer.GetEvents())

    expected_event_values = {
        'body': (
            'handle_get_value: AMPDevicesAgent attempting to get '
            '[InternationalMobileSubscriberIdentity2]'),
        'process_identifier': '69',
        'timestamp': '2021-10-13 07:57:42.869324'}

    self.CheckEventValues(storage_writer, events[6], expected_event_values)

    expected_event_values = {
        'body': (
            'spawn_xpc_service_block_invoke: description of xpc reply: '
            '<dictionary: 0x2029c5070> { count = 1, transaction: 0, voucher'
            ' = 0x0, contents =	"XPCErrorDescription" => <string: '
            '0x2029c5230> { length = 22, contents = "Connection '
            'interrupted" }}'),
        'process_identifier': '69',
        'timestamp': '2021-10-13 07:57:42.950704'}

    self.CheckEventValues(storage_writer, events[99], expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the attribute container store interface."""

import unittest

from plaso.containers import events
from plaso.storage import interface

from tests.storage import test_lib


class BaseStoreTest(test_lib.StorageTestCase):
  """Tests for the attribute container store interface."""

  # pylint: disable=protected-access

  # TODO: add tests for _DeserializeAttributeContainer method

  def testGetAttributeContainerNextSequenceNumber(self):
    """Tests the _GetAttributeContainerNextSequenceNumber function."""
    event_data_stream = events.EventDataStream()

    test_store = interface.BaseStore()

    sequence_number = test_store._GetAttributeContainerNextSequenceNumber(
        event_data_stream.CONTAINER_TYPE)
    self.assertEqual(sequence_number, 1)

    sequence_number = test_store._GetAttributeContainerNextSequenceNumber(
        event_data_stream.CONTAINER_TYPE)
    self.assertEqual(sequence_number, 2)

  # TODO: add tests for _GetAttributeContainerSchema method
  # TODO: add tests for _SerializeAttributeContainer method
  # TODO: add tests for _SetAttributeContainerNextSequenceNumber method

  def testSetSerializersProfiler(self):
    """Tests the SetSerializersProfiler function."""
    test_store = interface.BaseStore()
    test_store.SetSerializersProfiler(None)

  def testSetStorageProfiler(self):
    """Tests the SetStorageProfiler function."""
    test_store = interface.BaseStore()
    test_store.SetStorageProfiler(None)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the BagMRU Windows Registry plugin."""

import unittest

from plaso.parsers.winreg_plugins import bagmru

from tests.parsers.winreg_plugins import test_lib


class TestBagMRUWindowsRegistryPlugin(test_lib.RegistryPluginTestCase):
  """Tests for the BagMRU plugin."""

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = bagmru.BagMRUWindowsRegistryPlugin()

    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\Shell\\BagMRU')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\ShellNoRoam\\'
        'BagMRU')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_CURRENT_USER\\Software\\Classes\\Local Settings\\Software\\'
        'Microsoft\\Windows\\Shell\\BagMRU')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_CURRENT_USER\\Software\\Classes\\Local Settings\\Software\\'
        'Microsoft\\Windows\\ShellNoRoam\\BagMRU')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_CURRENT_USER\\Local Settings\\Software\\Microsoft\\Windows\\'
        'Shell\\BagMRU')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_CURRENT_USER\\Local Settings\\Software\\Microsoft\\Windows\\'
        'ShellNoRoam\\BagMRU')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    self._AssertNotFiltersOnKeyPath(plugin, 'HKEY_LOCAL_MACHINE\\Bogus')

  def testProcess(self):
    """Tests the Process function."""
    plugin = bagmru.BagMRUWindowsRegistryPlugin()
    test_file_entry = self._GetTestFileEntry(['NTUSER.DAT'])
    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\ShellNoRoam\\BagMRU')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 15)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    events = list(storage_writer.GetEvents())

    expected_event_values = {
        'date_time': '2009-08-04 15:19:16.9977500',
        'data_type': 'windows:registry:bagmru',
        'entries': (
            'Index: 1 [MRU Value 0]: Shell item path: <My Computer>'),
        # This should just be the plugin name, as we're invoking it directly,
        # and not through the parser.
        'parser': plugin.NAME}

    self.CheckEventValues(storage_writer, events[0], expected_event_values)

    expected_event_values = {
        'date_time': '2009-08-04 15:19:10.6696250',
        'entries': (
            'Index: 1 [MRU Value 0]: Shell item path: <My Computer> C:\\')}

    self.CheckEventValues(storage_writer, events[1], expected_event_values)

    expected_event_values = {
        'date_time': '2009-08-04 15:19:16.9977500',
        'key_path': '{0:s}\\0\\0\\0\\0\\0'.format(key_path)}

    self.CheckEventValues(storage_writer, events[14], expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""JSON-L parser plugin for Azure application gateway access log files."""

from dfdatetime import semantic_time as dfdatetime_semantic_time
from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import jsonl_parser
from plaso.parsers.jsonl_plugins import interface


class AzureApplicationGatewayAccessEventData(events.EventData):
  """Azure application gateway access log event data.

  Attributes:
    client_ip (str): Client IP address of the request.
    client_port (int): Client TCP/UDP port for the request.
    client_response_time (int): Duration, in seconds, from the first byte of
        a client request to be processed up to the first byte sent as response
        to the client.
    host (str): Address listed in the host header of the request. If rewritten
        using header rewrite, contains the updated host name.
    http_method (str): HTTP method used by the request.
    http_status (int): HTTP status code returned to the client from application
        gateway.
    http_version (str): HTTP version of the request.
    instance_identifier (str): Application gateway instance that served
        the request.
    original_host (str): Original request host name.
    original_request_uri (str): Original request URL, including arguments.
    received_bytes (int): Size of packet received, in bytes.
    request_query (str): Server-Routed: Back-end pool instance that was sent
        the request. X-AzureApplicationGateway-LOG-ID: Correlation ID used for
        the request. It can be used to troubleshoot traffic issues on
        the back-end servers. SERVER-STATUS: HTTP response code that application
        gateway received from the back-end.
    request_uri (str): URI of the received request.
    sent_bytes (int): Size of packet sent, in bytes.
    server_response_latency (str): Latency of the response (in seconds) from
        the back-end server.
    server_routed (str): The back-end server that application gateway routes
        the request to.
    server_status (str): HTTP status code of the back-end server.
    ssl_cipher (str): Cipher suite being used for TLS communication.
    ssl_client_certificate_fingerprint (str): Fingerprint of the SSL client
        certificate.
    ssl_client_certificate_issuer_name (str): Name of the issuer of the SSL
        client certificate.
    ssl_client_verify (str): TODO.
    ssl_enabled (str): Whether communication to the back-end pools used TLS.
        Valid values are on and off.
    ssl_protocol (str): The SSL/TLS protocol used.
    time_taken (double): Duration, in seconds, that it takes for the first byte
        of a client request to be processed and its last-byte sent in
        the response to the client. It's important to note that the Time-Taken
        field usually includes the time that the request and response packets
        are traveling over the network.
    transaction_id (str): Unique identifier to correlate the request received
        from the client
    user_agent (str): User agent from the HTTP request header.
    waf_evaluation_time (str): Duration, in seconds, that it takes for
        the request to be processed by the WAF.
    waf_mode (str): Value can be either Detection or Prevention.
  """

  DATA_TYPE = 'azure:applicationgatewayaccess:entry'

  def __init__(self):
    """Initializes event data."""
    super(AzureApplicationGatewayAccessEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.client_ip = None
    self.client_port = None
    self.client_response_time = None
    self.host = None
    self.http_method = None
    self.http_status = None
    self.http_version = None
    self.instance_identifier = None
    self.original_host = None
    self.original_request_uri = None
    self.received_bytes = None
    self.request_query = None
    self.request_uri = None
    self.sent_bytes = None
    self.server_response_latency = None
    self.server_routed = None
    self.server_status = None
    self.ssl_cipher = None
    self.ssl_client_certificate_fingerprint = None
    self.ssl_client_certificate_issuer_name = None
    self.ssl_client_verify = None
    self.ssl_enabled = None
    self.ssl_protocol = None
    self.time_taken = None
    self.transaction_identifier = None
    self.user_agent = None
    self.waf_evaluation_time = None
    self.waf_mode = None


class AzureApplicationGatewayAccessLogJSONLPlugin(interface.JSONLPlugin):
  """JSON-L parser plugin for Azure application gateway access log files."""

  NAME = 'azure_application_gateway_access_log'
  DATA_FORMAT = 'Azure Application Gateway access log'

  _PROPERTIES = {
      'clientIP': 'client_ip',
      'clientPort': 'client_port',
      'clientResponseTime': 'client_response_time',
      'host': 'host',
      'httpMethod': 'http_method',
      'httpStatus': 'http_status',
      'httpVersion': 'http_version',
      'instanceId': 'instance_identifier',
      'originalHost': 'original_host',
      'originalRequestUriWithArgs': 'original_request_uri',
      'receivedBytes': 'received_bytes',
      'requestQuery': 'request_query',
      'requestUri': 'request_uri',
      'sentBytes': 'sent_bytes',
      'serverResponseLatency': 'server_response_latency',
      'serverRouted': 'server_routed',
      'serverStatus': 'server_status',
      'sslCipher': 'ssl_cipher',
      'sslClientCertificateFingerprint': 'ssl_client_certificate_fingerprint',
      'sslClientCertificateIssuerName': 'ssl_client_certificate_issuer_name',
      'sslClientVerify': 'ssl_client_verify',
      'sslEnabled': 'ssl_enabled',
      'sslProtocol': 'ssl_protocol',
      'timeTaken': 'time_taken',
      'transactionId': 'transaction_identifier',
      'userAgent': 'user_agent',
      'WAFEvaluationTime': 'waf_evaluation_time',
      'WAFMode': 'waf_mode'}

  def _ParseRecord(self, parser_mediator, json_dict):
    """Parses an Azure application gateway access log record.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      json_dict (dict): JSON dictionary of the log record.
    """
    timestamp = self._GetJSONValue(json_dict, 'timeStamp')
    if not timestamp:
      parser_mediator.ProduceExtractionWarning(
          'Timestamp value missing from application gateway access log event')

    properties_json_dict = json_dict.get('properties')
    if not properties_json_dict:
      parser_mediator.ProduceExtractionWarning(
          'Properties value missing from application gateway access log event')
      return

    event_data = AzureApplicationGatewayAccessEventData()
    for json_name, attribute_name in self._PROPERTIES.items():
      attribute_value = self._GetJSONValue(properties_json_dict, json_name)
      setattr(event_data, attribute_name, attribute_value)

    try:
      date_time = dfdatetime_time_elements.TimeElementsInMicroseconds()
      date_time.CopyFromStringISO8601(timestamp)
    except ValueError as exception:
      parser_mediator.ProduceExtractionWarning(
          'Unable to parse timestamp value: {0:s} with error: {1!s}'.format(
              timestamp, exception))
      date_time = dfdatetime_semantic_time.InvalidTime()

    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_RECORDED)
    parser_mediator.ProduceEventWithEventData(event, event_data)

  def CheckRequiredFormat(self, json_dict):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      json_dict (dict): JSON dictionary of the log record.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    operation_name = json_dict.get('operationName') or None
    properties = json_dict.get('properties') or None
    timestamp = json_dict.get('timeStamp') or None

    if (None in (operation_name, properties, timestamp) or
        operation_name != 'ApplicationGatewayAccess'):
      return False

    try:
      date_time = dfdatetime_time_elements.TimeElementsInMicroseconds()
      date_time.CopyFromStringISO8601(timestamp)
    except ValueError:
      return False

    return True


jsonl_parser.JSONLParser.RegisterPlugin(
    AzureApplicationGatewayAccessLogJSONLPlugin)



# -*- coding: utf-8 -*-
"""The artifact knowledge base object.

The knowledge base is filled by user provided input and the pre-processing
phase. It is intended to provide successive phases, like the parsing and
analysis phases, with essential information like the time zone and codepage
of the source data.
"""

import codecs
import pytz

from plaso.containers import artifacts
from plaso.engine import logger
from plaso.helpers.windows import time_zones


class KnowledgeBase(object):
  """The knowledge base."""

  _DEFAULT_ACTIVE_SESSION = '00000000000000000000000000000000'

  def __init__(self):
    """Initializes a knowledge base."""
    super(KnowledgeBase, self).__init__()
    self._active_session = self._DEFAULT_ACTIVE_SESSION
    self._available_time_zones = {}
    self._codepage = 'cp1252'
    self._environment_variables = {}
    self._hostnames = {}
    self._language = 'en-US'
    self._mount_path = None
    self._time_zone = pytz.UTC
    self._user_accounts = {}
    self._values = {}
    self._windows_eventlog_providers = {}

  @property
  def available_time_zones(self):
    """list[TimeZone]: available time zones of the current session."""
    return self._available_time_zones.get(self._active_session, {}).values()

  @property
  def codepage(self):
    """str: codepage of the current session."""
    return self.GetValue('codepage', default_value=self._codepage)

  @property
  def language(self):
    """str: language of the current session."""
    return self.GetValue('language', default_value=self._language)

  @property
  def timezone(self):
    """datetime.tzinfo: time zone of the current session."""
    return self._time_zone

  @property
  def user_accounts(self):
    """list[UserAccountArtifact]: user accounts of the current session."""
    return self._user_accounts.get(self._active_session, {}).values()

  @property
  def year(self):
    """int: year of the current session."""
    return self.GetValue('year', default_value=0)

  def AddAvailableTimeZone(self, time_zone):
    """Adds an available time zone.

    Args:
      time_zone (TimeZoneArtifact): time zone artifact.

    Raises:
      KeyError: if the time zone already exists.
    """
    if self._active_session not in self._available_time_zones:
      self._available_time_zones[self._active_session] = {}

    available_time_zones = self._available_time_zones[self._active_session]
    if time_zone.name in available_time_zones:
      raise KeyError('Time zone: {0:s} already exists.'.format(time_zone.name))

    available_time_zones[time_zone.name] = time_zone

  def AddEnvironmentVariable(self, environment_variable):
    """Adds an environment variable.

    Args:
      environment_variable (EnvironmentVariableArtifact): environment variable
          artifact.

    Raises:
      KeyError: if the environment variable already exists.
    """
    name = environment_variable.name.upper()
    if name in self._environment_variables:
      raise KeyError('Environment variable: {0:s} already exists.'.format(
          environment_variable.name))

    self._environment_variables[name] = environment_variable

  def AddUserAccount(self, user_account):
    """Adds an user account.

    Args:
      user_account (UserAccountArtifact): user account artifact.

    Raises:
      KeyError: if the user account already exists.
    """
    if self._active_session not in self._user_accounts:
      self._user_accounts[self._active_session] = {}

    user_accounts = self._user_accounts[self._active_session]
    if user_account.identifier in user_accounts:
      raise KeyError('User account: {0:s} already exists.'.format(
          user_account.identifier))

    user_accounts[user_account.identifier] = user_account

  def AddWindowsEventLogProvider(self, windows_eventlog_provider):
    """Adds a Windows Event Log provider.

    Args:
      windows_eventlog_provider (WindowsEventLogProviderArtifact): Windows
          Event Log provider.

    Raises:
      KeyError: if the Windows Event Log provider already exists.
    """
    log_source = windows_eventlog_provider.log_source
    if log_source in self._windows_eventlog_providers:
      raise KeyError('Windows Event Log provider: {0:s} already exists.'.format(
          log_source))

    # TODO: store on a per-volume basis?
    self._windows_eventlog_providers[log_source] = windows_eventlog_provider

  def GetEnvironmentVariable(self, name):
    """Retrieves an environment variable.

    Args:
      name (str): name of the environment variable.

    Returns:
      EnvironmentVariableArtifact: environment variable artifact or None
          if there was no value set for the given name.
    """
    name = name.upper()
    return self._environment_variables.get(name, None)

  def GetEnvironmentVariables(self):
    """Retrieves the environment variables.

    Returns:
      list[EnvironmentVariableArtifact]: environment variable artifacts.
    """
    return self._environment_variables.values()

  def GetHostname(self):
    """Retrieves the hostname related to the event.

    If the hostname is not stored in the event it is determined based
    on the preprocessing information that is stored inside the storage file.

    Returns:
      str: hostname.
    """
    hostname_artifact = self._hostnames.get(self._active_session, None)
    if not hostname_artifact:
      return ''

    return hostname_artifact.name or ''

  def GetSystemConfigurationArtifact(self):
    """Retrieves the knowledge base as a system configuration artifact.

    Returns:
      SystemConfigurationArtifact: system configuration artifact.
    """
    system_configuration = artifacts.SystemConfigurationArtifact()

    system_configuration.code_page = self.GetValue(
        'codepage', default_value=self._codepage)

    system_configuration.hostname = self._hostnames.get(
        self._active_session, None)

    system_configuration.keyboard_layout = self.GetValue('keyboard_layout')

    system_configuration.language = self._language

    system_configuration.operating_system = self.GetValue('operating_system')
    system_configuration.operating_system_product = self.GetValue(
        'operating_system_product')
    system_configuration.operating_system_version = self.GetValue(
        'operating_system_version')

    time_zone = self._time_zone.zone
    if isinstance(time_zone, bytes):
      time_zone = time_zone.decode('ascii')

    system_configuration.time_zone = time_zone

    available_time_zones = self._available_time_zones.get(
        self._active_session, {})
    # In Python 3 dict.values() returns a type dict_values, which will cause
    # the JSON serializer to raise a TypeError.
    system_configuration.available_time_zones = list(
        available_time_zones.values())

    user_accounts = self._user_accounts.get(self._active_session, {})
    # In Python 3 dict.values() returns a type dict_values, which will cause
    # the JSON serializer to raise a TypeError.
    system_configuration.user_accounts = list(user_accounts.values())

    return system_configuration

  def GetUsernameByIdentifier(self, user_identifier):
    """Retrieves the username based on an user identifier.

    Args:
      user_identifier (str): user identifier, either a UID or SID.

    Returns:
      str: username.
    """
    user_accounts = self._user_accounts.get(self._active_session, {})
    user_account = user_accounts.get(user_identifier, None)
    if not user_account:
      return ''

    return user_account.username or ''

  def GetUsernameForPath(self, path):
    """Retrieves a username for a specific path.

    This is determining if a specific path is within a user's directory and
    returning the username of the user if so.

    Args:
      path (str): path.

    Returns:
      str: username or None if the path does not appear to be within a user's
          directory.
    """
    path = path.lower()

    user_accounts = self._user_accounts.get(self._active_session, {})
    for user_account in user_accounts.values():
      if not user_account.user_directory:
        continue

      user_directory = user_account.user_directory.lower()
      if path.startswith(user_directory):
        return user_account.username

    return None

  def GetValue(self, identifier, default_value=None):
    """Retrieves a value by identifier.

    Args:
      identifier (str): case insensitive unique identifier for the value.
      default_value (object): default value.

    Returns:
      object: value or default value if not available.

    Raises:
      TypeError: if the identifier is not a string type.
    """
    if not isinstance(identifier, str):
      raise TypeError('Identifier not a string type.')

    identifier = identifier.lower()
    return self._values.get(identifier, default_value)

  def GetWindowsEventLogProvider(self, log_source):
    """Retrieves a Windows EventLog provider by log source.

    Args:
      log_source (str): EventLog source, such as "Application Error".

    Returns:
      WindowsEventLogProviderArtifact: Windows EventLog provider artifact or
          None if not available.
    """
    return self._windows_eventlog_providers.get(log_source, None)

  def GetWindowsEventLogProviders(self):
    """Retrieves the Windows EventLog providers.

    Returns:
      list[WindowsEventLogProviderArtifact]: Windows EventLog provider
          artifacts.
    """
    return self._windows_eventlog_providers.values()

  def HasUserAccounts(self):
    """Determines if the knowledge base contains user accounts.

    Returns:
      bool: True if the knowledge base contains user accounts.
    """
    return self._user_accounts.get(self._active_session, {}) != {}

  def ReadSystemConfigurationArtifact(self, system_configuration):
    """Reads the knowledge base values from a system configuration artifact.

    Note that this overwrites existing values in the knowledge base.

    Args:
      system_configuration (SystemConfigurationArtifact): system configuration
          artifact.
    """
    if not system_configuration:
      return

    if system_configuration.code_page:
      try:
        self.SetCodepage(system_configuration.code_page)
      except ValueError:
        logger.warning(
            'Unsupported codepage: {0:s}, defaulting to {1:s}'.format(
                system_configuration.code_page, self._codepage))

    self._hostnames[self._active_session] = system_configuration.hostname

    self.SetValue('keyboard_layout', system_configuration.keyboard_layout)

    if system_configuration.language:
      self.SetLanguage(system_configuration.language)

    self.SetValue('operating_system', system_configuration.operating_system)
    self.SetValue(
        'operating_system_product',
        system_configuration.operating_system_product)
    self.SetValue(
        'operating_system_version',
        system_configuration.operating_system_version)

    # Set the available time zones before the system time zone so that localized
    # time zone names can be mapped to their corresponding Python time zone.
    self._available_time_zones[self._active_session] = {
        time_zone.name: time_zone
        for time_zone in system_configuration.available_time_zones}

    if system_configuration.time_zone:
      try:
        self.SetTimeZone(system_configuration.time_zone)
      except ValueError:
        logger.warning(
            'Unsupported time zone: {0:s}, defaulting to {1:s}'.format(
                system_configuration.time_zone, self._time_zone.zone))

    self._user_accounts[self._active_session] = {
        user_account.identifier: user_account
        for user_account in system_configuration.user_accounts}

  def SetActiveSession(self, session_identifier):
    """Sets the active session.

    Args:
      session_identifier (str): session identifier where None represents
          the default active session.
    """
    self._active_session = session_identifier or self._DEFAULT_ACTIVE_SESSION

  def SetCodepage(self, codepage):
    """Sets the codepage.

    Args:
      codepage (str): codepage.

    Raises:
      ValueError: if the codepage is not supported.
    """
    try:
      codecs.getencoder(codepage)
      self._codepage = codepage
    except LookupError:
      raise ValueError('Unsupported codepage: {0:s}'.format(codepage))

  def SetEnvironmentVariable(self, environment_variable):
    """Sets an environment variable.

    Args:
      environment_variable (EnvironmentVariableArtifact): environment variable
          artifact.
    """
    name = environment_variable.name.upper()
    self._environment_variables[name] = environment_variable

  def SetHostname(self, hostname):
    """Sets a hostname.

    Args:
      hostname (HostnameArtifact): hostname artifact.
    """
    self._hostnames[self._active_session] = hostname

  def SetLanguage(self, language):
    """Sets the language.

    Args:
      language (str): language.
    """
    self._language = language

  def SetTimeZone(self, time_zone):
    """Sets the time zone.

    Args:
      time_zone (str): time zone.

    Raises:
      ValueError: if the time zone is not supported.
    """
    # Get the "normalized" name of a Windows time zone name.
    if time_zone.startswith('@tzres.dll,'):
      mui_form_time_zones = {
          time_zone_artifact.mui_form: time_zone_artifact.name
          for time_zone_artifact in self.available_time_zones}

      time_zone = mui_form_time_zones.get(time_zone, time_zone)
    else:
      localized_time_zones = {
          time_zone_artifact.localized_name: time_zone_artifact.name
          for time_zone_artifact in self.available_time_zones}

      time_zone = localized_time_zones.get(time_zone, time_zone)

    # Map a Windows time zone name to a Python time zone name.
    time_zone = time_zones.WINDOWS_TIME_ZONES.get(time_zone, time_zone)

    try:
      self._time_zone = pytz.timezone(time_zone)
    except pytz.UnknownTimeZoneError:
      raise ValueError('Unsupported time zone: {0!s}'.format(time_zone))

  def SetValue(self, identifier, value):
    """Sets a value by identifier.

    Args:
      identifier (str): case insensitive unique identifier for the value.
      value (object): value.

    Raises:
      TypeError: if the identifier is not a string type.
    """
    if not isinstance(identifier, str):
      raise TypeError('Identifier not a string type.')

    identifier = identifier.lower()
    self._values[identifier] = value



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Dropbox sync_history database plugin."""

import unittest

from plaso.lib import definitions
from plaso.parsers.sqlite_plugins import dropbox

from tests.parsers.sqlite_plugins import test_lib


class DropboxSyncHistoryPluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Dropbox sync_history database plugin."""

  def testProcess(self):
    """Tests the Process function on a Dropbox sync_history database file."""
    plugin = dropbox.DropboxSyncDatabasePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['sync_history.db'], plugin)

    number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
    self.assertEqual(number_of_events, 6)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    events = list(storage_writer.GetSortedEvents())

    expected_event_values = {
        'event_type': 'file',
        'file_event_type': 'add',
        'direction': 'upload',
        'file_identifier': 'XXXXXXXXXXXAAAAAAAAAGg',
        'local_path': '/home/useraa/Dropbox/loc1/create_local.txt',
        'date_time': '2022-02-17 10:57:18',
        'timestamp_desc': definitions.TIME_DESCRIPTION_RECORDED}

    self.CheckEventValues(storage_writer, events[0], expected_event_values)

    expected_event_values = {
        'event_type': 'file',
        'file_event_type': 'delete',
        'direction': 'upload',
        'file_identifier': 'XXXXXXXXXXXAAAAAAAAAKg',
        'local_path': '/home/useraa/Dropbox/loc1/.create_local.txt.swp',
        'date_time': '2022-02-17 10:57:19',
        'timestamp_desc': definitions.TIME_DESCRIPTION_RECORDED}

    self.CheckEventValues(storage_writer, events[1], expected_event_values)

    expected_event_values = {
        'event_type': 'file',
        'file_event_type': 'add',
        'direction': 'download',
        'file_identifier': 'XXXXXXXXXXXAAAAAAAAAKw',
        'local_path': '/home/useraa/Dropbox/web1/create_web.txt',
        'date_time': '2022-02-17 11:01:21',
        'timestamp_desc': definitions.TIME_DESCRIPTION_RECORDED}

    self.CheckEventValues(storage_writer, events[2], expected_event_values)

    expected_event_values = {
        'event_type': 'file',
        'file_event_type': 'delete',
        'direction': 'download',
        'file_identifier': 'XXXXXXXXXXXAAAAAAAAALA',
        'local_path': '/home/useraa/Dropbox/web2/create_web.txt',
        'date_time': '2022-02-17 11:04:03',
        'timestamp_desc': definitions.TIME_DESCRIPTION_RECORDED}

    self.CheckEventValues(storage_writer, events[3], expected_event_values)

    expected_event_values = {
        'event_type': 'file',
        'file_event_type': 'edit',
        'direction': 'download',
        'file_identifier': 'XXXXXXXXXXXAAAAAAAAALQ',
        'local_path': '/home/useraa/Dropbox/web2/Document.docx',
        'date_time': '2022-02-17 11:05:50',
        'timestamp_desc': definitions.TIME_DESCRIPTION_RECORDED}

    self.CheckEventValues(storage_writer, events[4], expected_event_values)

    expected_event_values = {
        'event_type': 'file',
        'file_event_type': 'add',
        'direction': 'download',
        'file_identifier': 'XXXXXXXXXXXAAAAAAAAALg',
        'local_path': '/home/useraa/Dropbox/web2/Untitled.gdoc',
        'date_time': '2022-02-17 11:06:34',
        'timestamp_desc': definitions.TIME_DESCRIPTION_RECORDED}

    self.CheckEventValues(storage_writer, events[5], expected_event_values)


if __name__ == '__main__':
  unittest.main()
